"""Core pipeline orchestrator - 核心流水线编排器.

编排 Task -> Sandbox -> Recorder -> Reward -> Export 全流程。
"""

import json
import logging
import time
from dataclasses import dataclass, field
from pathlib import Path
from typing import Any, Dict, List, Optional

from trajectoryhub.config import AgentConfig, PipelineConfig
from trajectoryhub.tasks import TaskInfo, TaskLoader

logger = logging.getLogger(__name__)

# ── 可选依赖 (try/except) ──────────────────────────────────────────

try:
    import agentsandbox  # noqa: F401

    _HAS_SANDBOX = True
except ImportError:
    _HAS_SANDBOX = False

try:
    from agentrecorder import Recorder
    from agentrecorder.adapters import get_adapter as _get_recorder_adapter

    _HAS_RECORDER = True
except ImportError:
    _HAS_RECORDER = False

    def _get_recorder_adapter(name: str):  # type: ignore[misc]
        return None

try:
    from agentreward.reward import RewardEngine

    _HAS_REWARD = True
except ImportError:
    _HAS_REWARD = False

try:
    from knowlyrcore.domain import get_domain_profile

    _HAS_CORE_DOMAIN = True
except ImportError:
    _HAS_CORE_DOMAIN = False

    def get_domain_profile(domain: str):  # type: ignore[misc]
        return None


@dataclass
class Trajectory:
    """单条执行轨迹.

    Attributes:
        task_id: 任务 ID
        agent_framework: Agent 框架名
        agent_model: Agent 使用的模型
        steps: 执行步骤列表
        total_steps: 总步数
        success: 任务是否成功完成
        reward: 综合 Reward 分数
        step_rewards: 每步的 Reward 分数
        duration_seconds: 执行时长 (秒)
        metadata: 额外元数据
    """

    task_id: str = ""
    agent_framework: str = ""
    agent_model: str = ""
    steps: List[Dict[str, Any]] = field(default_factory=list)
    total_steps: int = 0
    success: bool = False
    reward: float = 0.0
    step_rewards: List[float] = field(default_factory=list)
    duration_seconds: float = 0.0
    metadata: Dict[str, Any] = field(default_factory=dict)


@dataclass
class PipelineResult:
    """Pipeline 执行结果.

    Attributes:
        total_tasks: 总任务数
        completed: 完成数
        failed: 失败数
        trajectories_path: 轨迹输出路径
        preferences_path: 偏好对输出路径
        quality_report_path: 质检报告路径
        duration_seconds: 总执行时长 (秒)
    """

    total_tasks: int = 0
    completed: int = 0
    failed: int = 0
    trajectories_path: Optional[str] = None
    preferences_path: Optional[str] = None
    quality_report_path: Optional[str] = None
    duration_seconds: float = 0.0


class Pipeline:
    """核心流水线编排器.

    串联整个 Agent 轨迹数据生产流水线：
    1. 加载任务 (Task)
    2. 对每个任务 x 每个 Agent：创建沙箱 -> 运行 Agent -> 录制轨迹 -> 打分
    3. 构建偏好对 (同任务的多条轨迹按 reward 排序)
    4. 运行数据质检 (data-check)
    5. 返回 PipelineResult

    Usage:
        config = PipelineConfig(
            task_source=TaskSource(path="tasks.jsonl"),
            agents=[
                AgentConfig(framework="openhands", model="claude-sonnet-4-20250514"),
                AgentConfig(framework="openhands", model="gpt-4o"),
            ],
            output_dir="./output",
        )

        pipeline = Pipeline(config)
        result = pipeline.run()
        print(f"完成: {result.completed}/{result.total_tasks}")
    """

    def __init__(self, config: PipelineConfig) -> None:
        self.config = config
        self._checkpoint_data: Dict[str, Any] = {}

    def run(self) -> PipelineResult:
        """运行完整 Pipeline.

        完整流程：
        1. 加载任务列表
        2. 对每个 (task, agent) 组合执行 run_single
        3. 构建偏好对 (同任务多轨迹按 reward 排序)
        4. 运行质检
        5. 保存结果

        Returns:
            PipelineResult: 执行结果汇总
        """
        start_time = time.time()
        output_dir = Path(self.config.output_dir)
        output_dir.mkdir(parents=True, exist_ok=True)

        # Step 1: 加载任务
        tasks = self._load_tasks()
        logger.info("加载 %d 个任务，%d 个 agent 配置", len(tasks), len(self.config.agents))

        # Step 2: 对每个 task x agent 执行
        all_trajectories: List[Trajectory] = []
        completed = 0
        failed = 0
        total_combinations = len(tasks) * len(self.config.agents)

        for task in tasks:
            for agent_config in self.config.agents:
                try:
                    trajectory = self.run_single(task, agent_config)
                    all_trajectories.append(trajectory)
                    completed += 1
                except Exception:
                    failed += 1
                    logger.exception(
                        "任务执行失败: task=%s, agent=%s/%s",
                        task.task_id, agent_config.framework, agent_config.model,
                    )

                if (completed + failed) % 10 == 0:
                    logger.info("进度: %d/%d (成功=%d, 失败=%d)",
                                completed + failed, total_combinations, completed, failed)

                # Checkpoint
                if (completed + failed) % self.config.checkpoint_interval == 0:
                    self._save_checkpoint(output_dir, all_trajectories, completed, failed)
                    logger.debug("Checkpoint 已保存: %d 条轨迹", len(all_trajectories))

        # Step 3: 保存轨迹
        trajectories_path = output_dir / "trajectories.jsonl"
        self._save_trajectories(trajectories_path, all_trajectories)
        logger.info("轨迹已保存: %s (%d 条)", trajectories_path, len(all_trajectories))

        # Step 4: 构建偏好对
        preferences_path = output_dir / "preferences.jsonl"
        self._build_preference_pairs(all_trajectories, preferences_path)

        # Step 5: 运行质检
        quality_report_path = output_dir / "quality_report.json"
        self._run_quality_check(trajectories_path, quality_report_path)

        duration = time.time() - start_time
        logger.info("Pipeline 完成: %d/%d 成功, 耗时 %.1fs", completed, len(tasks), duration)

        return PipelineResult(
            total_tasks=len(tasks),
            completed=completed,
            failed=failed,
            trajectories_path=str(trajectories_path),
            preferences_path=str(preferences_path),
            quality_report_path=str(quality_report_path),
            duration_seconds=duration,
        )

    def run_single(self, task: TaskInfo, agent_config: AgentConfig) -> Trajectory:
        """运行单个任务.

        流程：
        1. 创建沙箱环境 (knowlyr-sandbox)
        2. 在沙箱中运行 Agent (按 agent_config 配置)
        3. 录制执行轨迹 (knowlyr-recorder)
        4. 计算 Reward (knowlyr-reward)
        5. 返回 Trajectory

        当 Agent 框架未安装时返回空轨迹。
        如果只有日志文件而无需运行 Agent，请使用 run_from_log()。

        Args:
            task: 要执行的任务
            agent_config: Agent 配置

        Returns:
            Trajectory: 执行轨迹及其 reward
        """
        start_time = time.time()

        # 当前 Agent 框架执行部分仍需外部集成
        # 如需处理已有日志，请使用 run_from_log()
        logger.debug(
            "run_single: task=%s, agent=%s/%s (Agent 执行需外部集成)",
            task.task_id, agent_config.framework, agent_config.model,
        )

        return Trajectory(
            task_id=task.task_id,
            agent_framework=agent_config.framework,
            agent_model=agent_config.model,
            steps=[],
            total_steps=0,
            success=False,
            reward=0.0,
            step_rewards=[],
            duration_seconds=time.time() - start_time,
            metadata={
                "task_description": task.description,
                "task_type": task.type,
            },
        )

    def run_from_log(
        self,
        log_path: str | Path,
        framework: str,
        task: Optional[TaskInfo] = None,
    ) -> Trajectory:
        """从已有日志文件生成带评分的轨迹.

        这是处理已有 Agent 日志的主要入口。流程：
        1. 使用 recorder 适配器解析日志 → 标准 Trajectory
        2. 使用 reward engine 计算分数
        3. 合并为 hub Trajectory 返回

        Args:
            log_path: Agent 日志文件路径。
            framework: Agent 框架名 (openhands / sweagent)。
            task: 可选的任务信息。不传则从日志中自动提取。

        Returns:
            Trajectory: 带 reward 评分的标准轨迹。

        Raises:
            RuntimeError: recorder 或 reward 未安装。
            ValueError: 不支持的框架类型。
        """
        start_time = time.time()
        log_path = Path(log_path)

        # Step 1: 解析日志
        recorder_traj = self._parse_log(log_path, framework)
        logger.info("日志解析完成: %d 步 (%s)", len(recorder_traj.steps), framework)

        # 合并 task 信息
        if task is not None:
            recorder_traj.task = task

        # Step 2: Reward 评分
        reward_result = self._score_trajectory(recorder_traj)

        duration = time.time() - start_time

        # Step 3: 转换为 hub Trajectory
        return self._recorder_to_hub_trajectory(
            recorder_traj, reward_result, duration,
        )

    def run_batch_from_logs(
        self,
        log_dir: str | Path,
        framework: str,
        pattern: str = "*",
    ) -> List[Trajectory]:
        """批量处理日志目录.

        Args:
            log_dir: 包含日志文件的目录。
            framework: Agent 框架名。
            pattern: 文件匹配模式。

        Returns:
            Trajectory 列表。
        """
        if not _HAS_RECORDER:
            raise RuntimeError("批量处理需要安装 knowlyr-recorder: pip install knowlyr-recorder")

        adapter = _get_recorder_adapter(framework)
        if adapter is None:
            raise ValueError(f"不支持的框架: {framework}")

        recorder = Recorder(adapter)
        recorder_trajs = recorder.convert_batch(str(log_dir), pattern)

        trajectories = []
        for rtraj in recorder_trajs:
            reward_result = self._score_trajectory(rtraj)
            traj = self._recorder_to_hub_trajectory(rtraj, reward_result, 0.0)
            trajectories.append(traj)

        logger.info("批量处理完成: %d 条轨迹", len(trajectories))
        return trajectories

    def resume(self, checkpoint_path: str) -> PipelineResult:
        """从 checkpoint 恢复执行.

        读取 checkpoint 文件中已完成的任务，跳过这些任务后继续执行剩余任务。

        Args:
            checkpoint_path: checkpoint 文件路径

        Returns:
            PipelineResult: 执行结果汇总
        """
        checkpoint_file = Path(checkpoint_path)
        if not checkpoint_file.exists():
            raise FileNotFoundError(f"Checkpoint 文件不存在: {checkpoint_path}")

        with open(checkpoint_file, "r", encoding="utf-8") as f:
            self._checkpoint_data = json.load(f)

        # 获取已完成的 task_id 列表
        completed_ids = set(self._checkpoint_data.get("completed_task_ids", []))

        # 加载任务并过滤已完成的
        all_tasks = self._load_tasks()
        remaining_tasks = [t for t in all_tasks if t.task_id not in completed_ids]
        logger.info("从 checkpoint 恢复: 已完成 %d, 剩余 %d 个任务",
                     len(completed_ids), len(remaining_tasks))

        # 加载已有轨迹
        existing_trajectories: List[Trajectory] = []
        existing_path = self._checkpoint_data.get("trajectories_path")
        if existing_path and Path(existing_path).exists():
            existing_trajectories = self._load_trajectories(Path(existing_path))

        start_time = time.time()
        output_dir = Path(self.config.output_dir)
        output_dir.mkdir(parents=True, exist_ok=True)

        all_trajectories = list(existing_trajectories)
        completed = self._checkpoint_data.get("completed", 0)
        failed = self._checkpoint_data.get("failed", 0)

        for task in remaining_tasks:
            for agent_config in self.config.agents:
                try:
                    trajectory = self.run_single(task, agent_config)
                    all_trajectories.append(trajectory)
                    completed += 1
                except Exception:
                    failed += 1
                    logger.exception(
                        "任务执行失败 (resume): task=%s, agent=%s/%s",
                        task.task_id, agent_config.framework, agent_config.model,
                    )

        # 保存最终结果
        trajectories_path = output_dir / "trajectories.jsonl"
        self._save_trajectories(trajectories_path, all_trajectories)

        preferences_path = output_dir / "preferences.jsonl"
        self._build_preference_pairs(all_trajectories, preferences_path)

        quality_report_path = output_dir / "quality_report.json"
        self._run_quality_check(trajectories_path, quality_report_path)

        duration = time.time() - start_time

        return PipelineResult(
            total_tasks=len(all_tasks),
            completed=completed,
            failed=failed,
            trajectories_path=str(trajectories_path),
            preferences_path=str(preferences_path),
            quality_report_path=str(quality_report_path),
            duration_seconds=duration,
        )

    # ------------------------------------------------------------------
    # 集成方法 — recorder / reward
    # ------------------------------------------------------------------

    def _parse_log(self, log_path: Path, framework: str):
        """使用 recorder 适配器解析日志.

        Returns:
            agentrecorder.schema.Trajectory (recorder 格式)
        """
        if not _HAS_RECORDER:
            raise RuntimeError(
                "日志解析需要安装 knowlyr-recorder: pip install knowlyr-recorder"
            )

        adapter = _get_recorder_adapter(framework)
        if adapter is None:
            raise ValueError(f"不支持的框架: {framework}")

        recorder = Recorder(adapter)
        return recorder.convert(str(log_path))

    def _score_trajectory(self, recorder_traj) -> Optional[dict]:
        """使用 reward engine 评分.

        Args:
            recorder_traj: agentrecorder.schema.Trajectory

        Returns:
            RewardResult 或 None (reward 未安装时)
        """
        if not _HAS_REWARD:
            logger.debug("knowlyr-reward 未安装，跳过评分")
            return None

        # 传递 domain profile 给 RewardEngine
        profile = None
        if _HAS_CORE_DOMAIN:
            domain = self.config.domain or "coding"
            profile = get_domain_profile(domain)

        engine = RewardEngine(profile=profile)

        # 将 recorder 格式转换为 reward engine 输入
        steps = [
            {
                "tool": step.tool_call.name,
                "params": step.tool_call.parameters,
                "output": step.tool_result.output if step.tool_result else "",
            }
            for step in recorder_traj.steps
        ]

        outcome = {
            "success": recorder_traj.outcome.success,
            "tests_passed": recorder_traj.outcome.tests_passed,
            "tests_total": (
                recorder_traj.outcome.tests_passed + recorder_traj.outcome.tests_failed
            ),
        }

        result = engine.score({
            "task": recorder_traj.task.description,
            "steps": steps,
            "outcome": outcome,
        })

        return result

    def _recorder_to_hub_trajectory(
        self,
        recorder_traj,
        reward_result,
        duration: float,
    ) -> Trajectory:
        """将 recorder Trajectory + reward 结果合并为 hub Trajectory."""
        # 将 recorder steps 转为 dict 列表
        steps_dicts = []
        for step in recorder_traj.steps:
            steps_dicts.append({
                "thought": step.thought,
                "tool": step.tool_call.name,
                "params": step.tool_call.parameters,
                "output": step.tool_result.output if step.tool_result else "",
                "exit_code": step.tool_result.exit_code if step.tool_result else 0,
            })

        # 提取 reward
        total_reward = 0.0
        step_rewards = []
        if reward_result is not None:
            total_reward = reward_result.total_score
            step_rewards = [sr.total_score for sr in reward_result.step_rewards]

        return Trajectory(
            task_id=recorder_traj.task.task_id,
            agent_framework=recorder_traj.agent,
            agent_model=recorder_traj.model,
            steps=steps_dicts,
            total_steps=len(steps_dicts),
            success=recorder_traj.outcome.success,
            reward=total_reward,
            step_rewards=step_rewards,
            duration_seconds=duration,
            metadata={
                "task_description": recorder_traj.task.description,
                "outcome_tests_passed": recorder_traj.outcome.tests_passed,
                "outcome_tests_failed": recorder_traj.outcome.tests_failed,
                "outcome_total_tokens": recorder_traj.outcome.total_tokens,
                **recorder_traj.metadata,
            },
        )

    # ------------------------------------------------------------------
    # 内部方法
    # ------------------------------------------------------------------

    def _load_tasks(self) -> List[TaskInfo]:
        """根据配置加载任务列表."""
        source = self.config.task_source
        loader = TaskLoader()

        if source.source_type == "swebench" and source.dataset_name:
            tasks = loader.load_from_swebench(
                dataset_name=source.dataset_name,
                split=source.split,
                limit=source.limit,
            )
        elif source.path:
            tasks = loader.load_from_jsonl(source.path)
        else:
            return []

        # 应用过滤条件
        if source.filters:
            tasks = loader.filter_tasks(
                tasks,
                language=source.filters.get("language"),
                difficulty=source.filters.get("difficulty"),
                task_type=source.filters.get("type"),
            )

        if source.limit and len(tasks) > source.limit:
            tasks = tasks[: source.limit]

        return tasks

    def _save_trajectories(self, path: Path, trajectories: List[Trajectory]) -> None:
        """将轨迹列表保存为 JSONL."""
        path.parent.mkdir(parents=True, exist_ok=True)
        with open(path, "w", encoding="utf-8") as f:
            for traj in trajectories:
                line = json.dumps(
                    {
                        "task_id": traj.task_id,
                        "agent_framework": traj.agent_framework,
                        "agent_model": traj.agent_model,
                        "steps": traj.steps,
                        "total_steps": traj.total_steps,
                        "success": traj.success,
                        "reward": traj.reward,
                        "step_rewards": traj.step_rewards,
                        "duration_seconds": traj.duration_seconds,
                        "metadata": traj.metadata,
                    },
                    ensure_ascii=False,
                )
                f.write(line + "\n")

    def _load_trajectories(self, path: Path) -> List[Trajectory]:
        """从 JSONL 文件加载轨迹列表."""
        trajectories = []
        with open(path, "r", encoding="utf-8") as f:
            for line in f:
                line = line.strip()
                if not line:
                    continue
                data = json.loads(line)
                trajectories.append(
                    Trajectory(
                        task_id=data.get("task_id", ""),
                        agent_framework=data.get("agent_framework", ""),
                        agent_model=data.get("agent_model", ""),
                        steps=data.get("steps", []),
                        total_steps=data.get("total_steps", 0),
                        success=data.get("success", False),
                        reward=data.get("reward", 0.0),
                        step_rewards=data.get("step_rewards", []),
                        duration_seconds=data.get("duration_seconds", 0.0),
                        metadata=data.get("metadata", {}),
                    )
                )
        return trajectories

    def _build_preference_pairs(
        self, trajectories: List[Trajectory], output_path: Path
    ) -> None:
        """构建偏好对.

        同一任务的多条轨迹按 reward 排序，形成 (chosen, rejected) 对。
        """
        # 按 task_id 分组
        task_groups: Dict[str, List[Trajectory]] = {}
        for traj in trajectories:
            if traj.task_id not in task_groups:
                task_groups[traj.task_id] = []
            task_groups[traj.task_id].append(traj)

        output_path.parent.mkdir(parents=True, exist_ok=True)
        with open(output_path, "w", encoding="utf-8") as f:
            for task_id, group in task_groups.items():
                if len(group) < 2:
                    continue

                # 按 reward 排序
                sorted_group = sorted(group, key=lambda t: t.reward, reverse=True)

                # 两两配对: 高 reward 为 chosen, 低 reward 为 rejected
                for i in range(len(sorted_group) - 1):
                    chosen = sorted_group[i]
                    rejected = sorted_group[i + 1]

                    if chosen.reward == rejected.reward:
                        continue

                    pair = {
                        "task_id": task_id,
                        "chosen": {
                            "agent_framework": chosen.agent_framework,
                            "agent_model": chosen.agent_model,
                            "steps": chosen.steps,
                            "reward": chosen.reward,
                            "success": chosen.success,
                        },
                        "rejected": {
                            "agent_framework": rejected.agent_framework,
                            "agent_model": rejected.agent_model,
                            "steps": rejected.steps,
                            "reward": rejected.reward,
                            "success": rejected.success,
                        },
                        "reward_margin": chosen.reward - rejected.reward,
                    }
                    f.write(json.dumps(pair, ensure_ascii=False) + "\n")

    def _run_quality_check(self, trajectories_path: Path, report_path: Path) -> None:
        """运行数据质检.

        TODO: 接入 data-check (knowlyr-datacheck) 进行质检。
        当前生成一个占位报告。
        """
        report = {
            "status": "pending",
            "message": "质检功能待接入 knowlyr-datacheck",
            "trajectories_path": str(trajectories_path),
        }
        report_path.parent.mkdir(parents=True, exist_ok=True)
        with open(report_path, "w", encoding="utf-8") as f:
            json.dump(report, f, ensure_ascii=False, indent=2)

    def _save_checkpoint(
        self,
        output_dir: Path,
        trajectories: List[Trajectory],
        completed: int,
        failed: int,
    ) -> None:
        """保存 checkpoint 以支持断点恢复."""
        checkpoint_path = output_dir / "checkpoint.json"
        trajectories_path = output_dir / "trajectories_checkpoint.jsonl"

        self._save_trajectories(trajectories_path, trajectories)

        checkpoint = {
            "completed": completed,
            "failed": failed,
            "completed_task_ids": list({t.task_id for t in trajectories}),
            "trajectories_path": str(trajectories_path),
        }
        with open(checkpoint_path, "w", encoding="utf-8") as f:
            json.dump(checkpoint, f, ensure_ascii=False, indent=2)



"""Task loading and management - 任务加载与管理."""

import json
from pathlib import Path
from typing import Any, Dict, List, Optional

from knowlyrcore import TaskInfo


class TaskLoader:
    """任务加载器.

    支持从多种来源加载任务：
    - JSONL 文件 (本地任务定义)
    - SWE-bench 数据集 (通过 HuggingFace datasets 加载)

    Usage:
        loader = TaskLoader()

        # 从本地 JSONL 加载
        tasks = loader.load_from_jsonl("tasks.jsonl")

        # 从 SWE-bench 加载
        tasks = loader.load_from_swebench("princeton-nlp/SWE-bench_Verified", limit=100)

        # 过滤任务
        python_tasks = loader.filter_tasks(tasks, language="python", difficulty="medium")
    """

    def load_from_jsonl(self, path: str) -> List[TaskInfo]:
        """从 JSONL 文件加载任务.

        JSONL 格式 (每行一个 JSON):
        ```json
        {
            "task_id": "repo__issue-123",
            "description": "Fix the bug in parser module",
            "type": "bug_fix",
            "language": "python",
            "difficulty": "medium",
            "repo": "owner/repo",
            "base_commit": "abc123",
            "test_command": "pytest tests/test_parser.py"
        }
        ```

        Args:
            path: JSONL 文件路径

        Returns:
            List[TaskInfo]: 加载的任务列表
        """
        tasks = []
        file_path = Path(path)

        if not file_path.exists():
            raise FileNotFoundError(f"任务文件不存在: {path}")

        with open(file_path, "r", encoding="utf-8") as f:
            for line_num, line in enumerate(f, 1):
                line = line.strip()
                if not line:
                    continue
                try:
                    data = json.loads(line)
                    task = TaskInfo(
                        task_id=data.get("task_id", f"task_{line_num}"),
                        description=data.get("description", ""),
                        type=data.get("type", "bug_fix"),
                        language=data.get("language", "python"),
                        difficulty=data.get("difficulty", "medium"),
                        repo=data.get("repo", ""),
                        base_commit=data.get("base_commit", ""),
                        test_command=data.get("test_command", ""),
                        metadata=data.get("metadata", {}),
                    )
                    tasks.append(task)
                except json.JSONDecodeError:
                    continue

        return tasks

    def load_from_swebench(
        self,
        dataset_name: str = "princeton-nlp/SWE-bench_Verified",
        split: str = "test",
        limit: Optional[int] = None,
    ) -> List[TaskInfo]:
        """从 SWE-bench 数据集加载任务.

        需要安装 datasets 库: pip install datasets

        Args:
            dataset_name: HuggingFace 数据集名称
            split: 数据集分割 (test / train / dev)
            limit: 限制加载数量

        Returns:
            List[TaskInfo]: 加载的任务列表
        """
        try:
            from datasets import load_dataset
        except ImportError:
            raise ImportError(
                "加载 SWE-bench 需要 datasets 库。请运行: pip install datasets"
            )

        dataset = load_dataset(dataset_name, split=split)

        tasks = []
        for i, item in enumerate(dataset):
            if limit and i >= limit:
                break

            task = TaskInfo(
                task_id=item.get("instance_id", f"swebench_{i}"),
                description=item.get("problem_statement", ""),
                type="bug_fix",
                language="python",
                difficulty=self._estimate_difficulty(item),
                repo=item.get("repo", ""),
                base_commit=item.get("base_commit", ""),
                test_command=item.get("test_cmd", ""),
                metadata={
                    "source": dataset_name,
                    "patch": item.get("patch", ""),
                    "hints_text": item.get("hints_text", ""),
                    "created_at": item.get("created_at", ""),
                    "version": item.get("version", ""),
                },
            )
            tasks.append(task)

        return tasks

    def filter_tasks(
        self,
        tasks: List[TaskInfo],
        language: Optional[str] = None,
        difficulty: Optional[str] = None,
        task_type: Optional[str] = None,
    ) -> List[TaskInfo]:
        """按条件过滤任务.

        Args:
            tasks: 任务列表
            language: 按编程语言过滤
            difficulty: 按难度过滤
            task_type: 按任务类型过滤

        Returns:
            List[TaskInfo]: 过滤后的任务列表
        """
        filtered = tasks

        if language:
            filtered = [t for t in filtered if t.language.lower() == language.lower()]

        if difficulty:
            filtered = [t for t in filtered if t.difficulty.lower() == difficulty.lower()]

        if task_type:
            filtered = [t for t in filtered if t.type.lower() == task_type.lower()]

        return filtered

    def _estimate_difficulty(self, item: Dict[str, Any]) -> str:
        """根据 SWE-bench 数据估算任务难度.

        简单启发式: 按 patch 大小估算。
        """
        patch = item.get("patch", "")
        patch_lines = len(patch.split("\n"))

        if patch_lines < 10:
            return "easy"
        elif patch_lines < 50:
            return "medium"
        else:
            return "hard"



"""MCP Server - Model Context Protocol 服务."""

import json
from pathlib import Path
from typing import Any, Dict, List

try:
    from mcp.server import Server
    from mcp.server.stdio import stdio_server
    from mcp.types import Tool, TextContent

    HAS_MCP = True
except ImportError:
    HAS_MCP = False

from trajectoryhub.config import PipelineConfig, TaskSource, AgentConfig
from trajectoryhub.pipeline import Pipeline
from trajectoryhub.exporter import DatasetExporter


def create_server() -> "Server":
    """创建 MCP 服务器实例."""
    if not HAS_MCP:
        raise ImportError("MCP 未安装。请运行: pip install knowlyr-hub[mcp]")

    server = Server("trajectoryhub")

    @server.list_tools()
    async def list_tools() -> List[Tool]:
        """列出可用的工具."""
        return [
            Tool(
                name="run_pipeline",
                description="运行完整的 Agent 轨迹数据 Pipeline (Task -> Sandbox -> Recorder -> Reward -> Export)",
                inputSchema={
                    "type": "object",
                    "properties": {
                        "task_source": {
                            "type": "string",
                            "description": "任务定义文件路径 (JSONL 格式)",
                        },
                        "agents": {
                            "type": "array",
                            "items": {
                                "type": "object",
                                "properties": {
                                    "framework": {
                                        "type": "string",
                                        "description": "Agent 框架 (openhands / sweagent)",
                                    },
                                    "model": {
                                        "type": "string",
                                        "description": "LLM 模型",
                                    },
                                },
                            },
                            "description": "Agent 配置列表",
                        },
                        "output_dir": {
                            "type": "string",
                            "description": "输出目录 (默认: ./output)",
                            "default": "./output",
                        },
                        "parallel_workers": {
                            "type": "integer",
                            "description": "并行工作进程数 (默认: 1)",
                            "default": 1,
                        },
                    },
                    "required": ["task_source"],
                },
            ),
            Tool(
                name="export_dataset",
                description="将轨迹数据导出为指定的训练格式 (SFT / DPO / Benchmark / HuggingFace)",
                inputSchema={
                    "type": "object",
                    "properties": {
                        "trajectories_path": {
                            "type": "string",
                            "description": "轨迹文件路径 (JSONL)",
                        },
                        "preferences_path": {
                            "type": "string",
                            "description": "偏好对文件路径 (JSONL，DPO 导出时必需)",
                        },
                        "format": {
                            "type": "string",
                            "enum": ["sft", "dpo", "benchmark", "huggingface"],
                            "description": "导出格式",
                        },
                        "output_path": {
                            "type": "string",
                            "description": "输出文件路径",
                        },
                        "repo_id": {
                            "type": "string",
                            "description": "HuggingFace 仓库 ID (仅 huggingface 格式)",
                        },
                    },
                    "required": ["trajectories_path", "format", "output_path"],
                },
            ),
            Tool(
                name="process_log",
                description="处理单个 Agent 日志文件，解析并评分生成标准轨迹",
                inputSchema={
                    "type": "object",
                    "properties": {
                        "log_path": {
                            "type": "string",
                            "description": "Agent 日志文件路径",
                        },
                        "framework": {
                            "type": "string",
                            "enum": ["openhands", "sweagent", "swe-agent"],
                            "description": "Agent 框架",
                        },
                        "output_dir": {
                            "type": "string",
                            "description": "输出目录 (默认: ./output)",
                            "default": "./output",
                        },
                    },
                    "required": ["log_path", "framework"],
                },
            ),
            Tool(
                name="process_logs_batch",
                description="批量处理 Agent 日志目录，解析并评分生成标准轨迹",
                inputSchema={
                    "type": "object",
                    "properties": {
                        "log_dir": {
                            "type": "string",
                            "description": "包含日志文件的目录",
                        },
                        "framework": {
                            "type": "string",
                            "enum": ["openhands", "sweagent", "swe-agent"],
                            "description": "Agent 框架",
                        },
                        "pattern": {
                            "type": "string",
                            "description": "文件匹配模式 (默认: *)",
                            "default": "*",
                        },
                        "output_dir": {
                            "type": "string",
                            "description": "输出目录 (默认: ./output)",
                            "default": "./output",
                        },
                    },
                    "required": ["log_dir", "framework"],
                },
            ),
            Tool(
                name="pipeline_status",
                description="查看 Pipeline 执行状态和进度",
                inputSchema={
                    "type": "object",
                    "properties": {
                        "output_dir": {
                            "type": "string",
                            "description": "Pipeline 输出目录",
                        },
                    },
                    "required": ["output_dir"],
                },
            ),
        ]

    @server.call_tool()
    async def call_tool(name: str, arguments: Dict[str, Any]) -> List[TextContent]:
        """调用工具."""

        if name == "run_pipeline":
            # 构建配置
            agents = []
            for agent_data in arguments.get("agents", [{"framework": "openhands"}]):
                agents.append(
                    AgentConfig(
                        framework=agent_data.get("framework", "openhands"),
                        model=agent_data.get("model", "claude-sonnet-4-20250514"),
                    )
                )

            config = PipelineConfig(
                task_source=TaskSource(
                    path=arguments["task_source"],
                    source_type="jsonl",
                ),
                agents=agents,
                output_dir=arguments.get("output_dir", "./output"),
                parallel_workers=arguments.get("parallel_workers", 1),
            )

            pipeline = Pipeline(config)
            result = pipeline.run()

            return [
                TextContent(
                    type="text",
                    text=f"Pipeline 执行完成:\n"
                    f"- 总任务数: {result.total_tasks}\n"
                    f"- 完成: {result.completed}\n"
                    f"- 失败: {result.failed}\n"
                    f"- 轨迹输出: {result.trajectories_path}\n"
                    f"- 偏好对输出: {result.preferences_path}\n"
                    f"- 质检报告: {result.quality_report_path}\n"
                    f"- 耗时: {result.duration_seconds:.1f}s",
                )
            ]

        elif name == "export_dataset":
            exporter = DatasetExporter(
                trajectories_dir=arguments["trajectories_path"],
                preferences_dir=arguments.get("preferences_path"),
            )

            export_format = arguments["format"]
            output_path = arguments["output_path"]

            if export_format == "sft":
                result = exporter.export_sft(output_path)
            elif export_format == "dpo":
                result = exporter.export_dpo(output_path)
            elif export_format == "benchmark":
                result = exporter.export_benchmark(output_path)
            elif export_format == "huggingface":
                repo_id = arguments.get("repo_id", "")
                result = exporter.export_huggingface(repo_id)
            else:
                return [TextContent(type="text", text=f"不支持的导出格式: {export_format}")]

            if result.success:
                return [
                    TextContent(
                        type="text",
                        text=f"导出成功:\n"
                        f"- 格式: {result.format}\n"
                        f"- 输出路径: {result.output_path}\n"
                        f"- 导出记录数: {result.total_records}",
                    )
                ]
            else:
                return [TextContent(type="text", text=f"导出失败: {result.error}")]

        elif name == "process_log":
            log_path = arguments["log_path"]
            framework = arguments["framework"]
            output_dir = arguments.get("output_dir", "./output")

            config = PipelineConfig(output_dir=output_dir)
            pipeline = Pipeline(config)

            try:
                traj = pipeline.run_from_log(log_path, framework)
            except (RuntimeError, ValueError) as e:
                return [TextContent(type="text", text=f"处理失败: {e}")]

            return [
                TextContent(
                    type="text",
                    text=f"日志处理完成:\n"
                    f"- 任务 ID: {traj.task_id}\n"
                    f"- 框架: {traj.agent_framework}\n"
                    f"- 模型: {traj.agent_model}\n"
                    f"- 步数: {traj.total_steps}\n"
                    f"- 成功: {traj.success}\n"
                    f"- Reward: {traj.reward:.3f}\n"
                    f"- 耗时: {traj.duration_seconds:.1f}s",
                )
            ]

        elif name == "process_logs_batch":
            log_dir = arguments["log_dir"]
            framework = arguments["framework"]
            pattern = arguments.get("pattern", "*")
            output_dir = arguments.get("output_dir", "./output")

            config = PipelineConfig(output_dir=output_dir)
            pipeline = Pipeline(config)

            try:
                trajectories = pipeline.run_batch_from_logs(log_dir, framework, pattern)
            except (RuntimeError, ValueError) as e:
                return [TextContent(type="text", text=f"批量处理失败: {e}")]

            if not trajectories:
                return [TextContent(type="text", text="没有找到匹配的日志文件。")]

            # 保存轨迹
            out_path = Path(output_dir)
            out_path.mkdir(parents=True, exist_ok=True)
            traj_path = out_path / "trajectories.jsonl"
            with open(traj_path, "w", encoding="utf-8") as f:
                for traj in trajectories:
                    f.write(json.dumps({
                        "task_id": traj.task_id,
                        "agent_framework": traj.agent_framework,
                        "agent_model": traj.agent_model,
                        "steps": traj.steps,
                        "total_steps": traj.total_steps,
                        "success": traj.success,
                        "reward": traj.reward,
                        "step_rewards": traj.step_rewards,
                        "duration_seconds": traj.duration_seconds,
                        "metadata": traj.metadata,
                    }, ensure_ascii=False) + "\n")

            success_count = sum(1 for t in trajectories if t.success)
            avg_reward = (
                sum(t.reward for t in trajectories) / len(trajectories)
                if trajectories else 0.0
            )

            return [
                TextContent(
                    type="text",
                    text=f"批量处理完成:\n"
                    f"- 轨迹数: {len(trajectories)}\n"
                    f"- 成功率: {success_count}/{len(trajectories)}\n"
                    f"- 平均 Reward: {avg_reward:.3f}\n"
                    f"- 输出: {traj_path}",
                )
            ]

        elif name == "pipeline_status":
            output_dir = Path(arguments["output_dir"])

            # 检查各个文件是否存在
            checkpoint_path = output_dir / "checkpoint.json"
            trajectories_path = output_dir / "trajectories.jsonl"
            preferences_path = output_dir / "preferences.jsonl"
            report_path = output_dir / "quality_report.json"

            status_parts = [f"Pipeline 状态 ({output_dir}):"]

            if checkpoint_path.exists():
                with open(checkpoint_path, "r", encoding="utf-8") as f:
                    checkpoint = json.load(f)
                status_parts.append(
                    f"- Checkpoint: 已完成 {checkpoint.get('completed', 0)} 个, "
                    f"失败 {checkpoint.get('failed', 0)} 个"
                )
            else:
                status_parts.append("- Checkpoint: 无")

            if trajectories_path.exists():
                with open(trajectories_path, "r", encoding="utf-8") as f:
                    count = sum(1 for _ in f)
                status_parts.append(f"- 轨迹文件: {count} 条")
            else:
                status_parts.append("- 轨迹文件: 未生成")

            if preferences_path.exists():
                with open(preferences_path, "r", encoding="utf-8") as f:
                    count = sum(1 for _ in f)
                status_parts.append(f"- 偏好对文件: {count} 对")
            else:
                status_parts.append("- 偏好对文件: 未生成")

            if report_path.exists():
                status_parts.append("- 质检报告: 已生成")
            else:
                status_parts.append("- 质检报告: 未生成")

            return [TextContent(type="text", text="\n".join(status_parts))]

        else:
            return [TextContent(type="text", text=f"未知工具: {name}")]

    return server


async def serve():
    """启动 MCP 服务器."""
    if not HAS_MCP:
        raise ImportError("MCP 未安装。请运行: pip install knowlyr-hub[mcp]")

    server = create_server()
    async with stdio_server() as (read_stream, write_stream):
        await server.run(read_stream, write_stream)


def main():
    """主入口."""
    import asyncio

    asyncio.run(serve())


if __name__ == "__main__":
    main()



"""Tests for DatasetExporter format conversion."""

import json
import tempfile
from pathlib import Path

from trajectoryhub.exporter import DatasetExporter, ExportResult


class TestExportResult:
    """Tests for ExportResult dataclass."""

    def test_default_result(self):
        """Test default export result."""
        result = ExportResult()
        assert result.success is False
        assert result.total_records == 0
        assert result.error is None

    def test_success_result(self):
        """Test successful export result."""
        result = ExportResult(
            success=True,
            output_path="/tmp/output.jsonl",
            total_records=100,
            format="sft",
        )
        assert result.success is True
        assert result.total_records == 100
        assert result.format == "sft"


class TestDatasetExporter:
    """Tests for DatasetExporter."""

    def _create_test_trajectories(self, tmp_dir: Path) -> Path:
        """创建测试用的轨迹文件."""
        traj_path = tmp_dir / "trajectories.jsonl"
        trajectories = [
            {
                "task_id": "task-001",
                "agent_framework": "openhands",
                "agent_model": "claude-sonnet-4-20250514",
                "steps": [
                    {"thought": "Read the file", "action": "file_read /test.py", "observation": "def test(): pass"},
                    {"thought": "Fix the bug", "action": "file_write /test.py", "observation": "File written"},
                ],
                "total_steps": 2,
                "success": True,
                "reward": 0.85,
                "step_rewards": [0.8, 0.9],
                "duration_seconds": 15.0,
                "metadata": {"task_description": "Fix the test function"},
            },
            {
                "task_id": "task-001",
                "agent_framework": "sweagent",
                "agent_model": "gpt-4o",
                "steps": [
                    {"thought": "Look at file", "action": "file_read /test.py", "observation": "def test(): pass"},
                ],
                "total_steps": 1,
                "success": False,
                "reward": 0.3,
                "step_rewards": [0.3],
                "duration_seconds": 8.0,
                "metadata": {"task_description": "Fix the test function"},
            },
        ]
        with open(traj_path, "w", encoding="utf-8") as f:
            for traj in trajectories:
                f.write(json.dumps(traj, ensure_ascii=False) + "\n")
        return traj_path

    def _create_test_preferences(self, tmp_dir: Path) -> Path:
        """创建测试用的偏好对文件."""
        pref_path = tmp_dir / "preferences.jsonl"
        preferences = [
            {
                "task_id": "task-001",
                "chosen": {
                    "agent_framework": "openhands",
                    "agent_model": "claude-sonnet-4-20250514",
                    "steps": [
                        {"thought": "Read the file", "action": "file_read /test.py", "observation": "content"},
                    ],
                    "reward": 0.85,
                    "success": True,
                },
                "rejected": {
                    "agent_framework": "sweagent",
                    "agent_model": "gpt-4o",
                    "steps": [
                        {"thought": "Look at file", "action": "file_read /test.py", "observation": "content"},
                    ],
                    "reward": 0.3,
                    "success": False,
                },
                "reward_margin": 0.55,
            }
        ]
        with open(pref_path, "w", encoding="utf-8") as f:
            for pref in preferences:
                f.write(json.dumps(pref, ensure_ascii=False) + "\n")
        return pref_path

    def test_export_sft(self):
        """Test SFT format export."""
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_path = Path(tmp_dir)
            traj_path = self._create_test_trajectories(tmp_path)
            output_path = tmp_path / "sft_output.jsonl"

            exporter = DatasetExporter(trajectories_dir=str(traj_path))
            result = exporter.export_sft(str(output_path))

            assert result.success is True
            assert result.format == "sft"
            assert result.total_records == 1  # 只有 1 条成功的

            # 验证输出格式
            with open(output_path, "r", encoding="utf-8") as f:
                records = [json.loads(line) for line in f if line.strip()]

            assert len(records) == 1
            record = records[0]
            assert "instruction" in record
            assert "response" in record
            assert "task_id" in record
            assert record["task_id"] == "task-001"
            assert record["reward"] == 0.85

    def test_export_dpo(self):
        """Test DPO format export."""
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_path = Path(tmp_dir)
            traj_path = self._create_test_trajectories(tmp_path)
            pref_path = self._create_test_preferences(tmp_path)
            output_path = tmp_path / "dpo_output.jsonl"

            exporter = DatasetExporter(
                trajectories_dir=str(traj_path),
                preferences_dir=str(pref_path),
            )
            result = exporter.export_dpo(str(output_path))

            assert result.success is True
            assert result.format == "dpo"
            assert result.total_records == 1

            # 验证输出格式
            with open(output_path, "r", encoding="utf-8") as f:
                records = [json.loads(line) for line in f if line.strip()]

            assert len(records) == 1
            record = records[0]
            assert "prompt" in record
            assert "chosen" in record
            assert "rejected" in record
            assert "reward_margin" in record
            assert record["reward_margin"] == 0.55

    def test_export_dpo_no_preferences(self):
        """Test DPO export fails gracefully without preferences file."""
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_path = Path(tmp_dir)
            traj_path = self._create_test_trajectories(tmp_path)

            exporter = DatasetExporter(trajectories_dir=str(traj_path))
            result = exporter.export_dpo(str(tmp_path / "dpo_output.jsonl"))

            assert result.success is False
            assert result.error is not None

    def test_export_benchmark(self):
        """Test benchmark format export."""
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_path = Path(tmp_dir)
            traj_path = self._create_test_trajectories(tmp_path)
            output_path = tmp_path / "benchmark_output.jsonl"

            exporter = DatasetExporter(trajectories_dir=str(traj_path))
            result = exporter.export_benchmark(str(output_path))

            assert result.success is True
            assert result.format == "benchmark"

    def test_generate_datacard(self):
        """Test dataset card generation."""
        with tempfile.TemporaryDirectory() as tmp_dir:
            tmp_path = Path(tmp_dir)
            traj_path = self._create_test_trajectories(tmp_path)

            exporter = DatasetExporter(trajectories_dir=str(traj_path))
            card = exporter.generate_datacard()

            assert "Agent Trajectory Dataset" in card
            assert "Total trajectories" in card
            assert "license: mit" in card

    def test_export_huggingface_without_lib(self):
        """无 huggingface_hub 时应返回安装提示."""
        import trajectoryhub.exporter as exp_mod
        original = exp_mod._HAS_HF
        try:
            exp_mod._HAS_HF = False
            with tempfile.TemporaryDirectory() as tmp_dir:
                tmp_path = Path(tmp_dir)
                traj_path = self._create_test_trajectories(tmp_path)

                exporter = DatasetExporter(trajectories_dir=str(traj_path))
                result = exporter.export_huggingface("test/repo")

                assert result.success is False
                assert "huggingface" in result.error.lower()
        finally:
            exp_mod._HAS_HF = original

    def test_export_huggingface_with_mock(self):
        """有 huggingface_hub 时应调用 API 上传."""
        from unittest.mock import MagicMock, patch
        import trajectoryhub.exporter as exp_mod

        original = exp_mod._HAS_HF
        try:
            exp_mod._HAS_HF = True
            mock_api = MagicMock()

            with tempfile.TemporaryDirectory() as tmp_dir:
                tmp_path = Path(tmp_dir)
                traj_path = self._create_test_trajectories(tmp_path)

                exporter = DatasetExporter(trajectories_dir=str(traj_path))

                with patch.object(exp_mod, "HfApi", return_value=mock_api, create=True):
                    result = exporter.export_huggingface("user/test-dataset")

                assert result.success is True
                assert result.format == "huggingface"
                assert result.total_records > 0
                assert "huggingface.co" in result.output_path
                mock_api.create_repo.assert_called_once_with(
                    "user/test-dataset", repo_type="dataset", exist_ok=True,
                )
                mock_api.upload_folder.assert_called_once()
        finally:
            exp_mod._HAS_HF = original



"""测试 TaskLoader 任务加载."""

import json

import pytest

from knowlyrcore import TaskInfo
from trajectoryhub.tasks import TaskLoader


def _write_tasks_jsonl(path, tasks=None):
    """写测试用任务文件."""
    if tasks is None:
        tasks = [
            {
                "task_id": "task-001",
                "description": "修复排序 bug",
                "type": "bug_fix",
                "language": "python",
                "difficulty": "easy",
                "repo": "org/repo",
                "base_commit": "abc123",
                "test_command": "pytest tests/test_sort.py",
            },
            {
                "task_id": "task-002",
                "description": "添加缓存功能",
                "type": "feature",
                "language": "python",
                "difficulty": "medium",
            },
            {
                "task_id": "task-003",
                "description": "Fix JS linting",
                "type": "bug_fix",
                "language": "javascript",
                "difficulty": "easy",
            },
        ]
    with open(path, "w", encoding="utf-8") as f:
        for t in tasks:
            f.write(json.dumps(t, ensure_ascii=False) + "\n")


class TestLoadFromJsonl:
    """测试 JSONL 加载."""

    def test_basic_load(self, tmp_path):
        """加载基本任务文件."""
        path = tmp_path / "tasks.jsonl"
        _write_tasks_jsonl(path)

        loader = TaskLoader()
        tasks = loader.load_from_jsonl(str(path))

        assert len(tasks) == 3
        assert all(isinstance(t, TaskInfo) for t in tasks)
        assert tasks[0].task_id == "task-001"
        assert tasks[0].description == "修复排序 bug"
        assert tasks[0].repo == "org/repo"

    def test_file_not_found(self):
        """文件不存在应抛出 FileNotFoundError."""
        loader = TaskLoader()
        with pytest.raises(FileNotFoundError, match="任务文件不存在"):
            loader.load_from_jsonl("/nonexistent/path.jsonl")

    def test_empty_lines_skipped(self, tmp_path):
        """空行应被跳过."""
        path = tmp_path / "tasks.jsonl"
        with open(path, "w") as f:
            f.write('{"task_id": "t1", "description": "test"}\n')
            f.write("\n")
            f.write('{"task_id": "t2", "description": "test2"}\n')

        loader = TaskLoader()
        tasks = loader.load_from_jsonl(str(path))
        assert len(tasks) == 2

    def test_invalid_json_skipped(self, tmp_path):
        """无效 JSON 行应被跳过."""
        path = tmp_path / "tasks.jsonl"
        with open(path, "w") as f:
            f.write('{"task_id": "t1", "description": "ok"}\n')
            f.write("not valid json\n")
            f.write('{"task_id": "t2", "description": "ok2"}\n')

        loader = TaskLoader()
        tasks = loader.load_from_jsonl(str(path))
        assert len(tasks) == 2

    def test_auto_task_id(self, tmp_path):
        """缺少 task_id 应自动生成."""
        path = tmp_path / "tasks.jsonl"
        with open(path, "w") as f:
            f.write('{"description": "no id task"}\n')

        loader = TaskLoader()
        tasks = loader.load_from_jsonl(str(path))
        assert tasks[0].task_id == "task_1"

    def test_default_fields(self, tmp_path):
        """缺少字段应用默认值."""
        path = tmp_path / "tasks.jsonl"
        with open(path, "w") as f:
            f.write('{"task_id": "t1"}\n')

        loader = TaskLoader()
        tasks = loader.load_from_jsonl(str(path))
        assert tasks[0].type == "bug_fix"
        assert tasks[0].language == "python"
        assert tasks[0].difficulty == "medium"


class TestFilterTasks:
    """测试任务过滤."""

    def _load_test_tasks(self, tmp_path):
        path = tmp_path / "tasks.jsonl"
        _write_tasks_jsonl(path)
        return TaskLoader().load_from_jsonl(str(path))

    def test_filter_by_language(self, tmp_path):
        """按语言过滤."""
        tasks = self._load_test_tasks(tmp_path)
        loader = TaskLoader()
        filtered = loader.filter_tasks(tasks, language="python")
        assert len(filtered) == 2
        assert all(t.language == "python" for t in filtered)

    def test_filter_by_difficulty(self, tmp_path):
        """按难度过滤."""
        tasks = self._load_test_tasks(tmp_path)
        loader = TaskLoader()
        filtered = loader.filter_tasks(tasks, difficulty="easy")
        assert len(filtered) == 2

    def test_filter_by_type(self, tmp_path):
        """按类型过滤."""
        tasks = self._load_test_tasks(tmp_path)
        loader = TaskLoader()
        filtered = loader.filter_tasks(tasks, task_type="feature")
        assert len(filtered) == 1
        assert filtered[0].task_id == "task-002"

    def test_filter_combined(self, tmp_path):
        """组合过滤."""
        tasks = self._load_test_tasks(tmp_path)
        loader = TaskLoader()
        filtered = loader.filter_tasks(tasks, language="python", difficulty="easy")
        assert len(filtered) == 1
        assert filtered[0].task_id == "task-001"

    def test_filter_no_match(self, tmp_path):
        """无匹配应返回空列表."""
        tasks = self._load_test_tasks(tmp_path)
        loader = TaskLoader()
        filtered = loader.filter_tasks(tasks, language="rust")
        assert len(filtered) == 0

    def test_filter_case_insensitive(self, tmp_path):
        """过滤应不区分大小写."""
        tasks = self._load_test_tasks(tmp_path)
        loader = TaskLoader()
        filtered = loader.filter_tasks(tasks, language="Python")
        assert len(filtered) == 2


class TestEstimateDifficulty:
    """测试难度估算."""

    def test_easy(self):
        """短 patch 应为 easy."""
        loader = TaskLoader()
        assert loader._estimate_difficulty({"patch": "a\nb\nc"}) == "easy"

    def test_medium(self):
        """中等 patch 应为 medium."""
        loader = TaskLoader()
        patch = "\n".join([f"line {i}" for i in range(30)])
        assert loader._estimate_difficulty({"patch": patch}) == "medium"

    def test_hard(self):
        """长 patch 应为 hard."""
        loader = TaskLoader()
        patch = "\n".join([f"line {i}" for i in range(60)])
        assert loader._estimate_difficulty({"patch": patch}) == "hard"

    def test_no_patch(self):
        """无 patch 应为 easy."""
        loader = TaskLoader()
        assert loader._estimate_difficulty({}) == "easy"



"""测试 Hub 配置模型."""

from trajectoryhub.config import (
    AgentConfig,
    PipelineConfig,
    RewardConfig,
    SandboxConfig,
    TaskSource,
)


class TestTaskSource:
    """测试任务来源配置."""

    def test_defaults(self):
        """默认值."""
        ts = TaskSource()
        assert ts.path is None
        assert ts.source_type == "jsonl"
        assert ts.split == "test"
        assert ts.limit is None
        assert ts.filters == {}

    def test_custom(self):
        """自定义配置."""
        ts = TaskSource(
            path="/data/tasks.jsonl",
            source_type="swebench",
            dataset_name="princeton-nlp/SWE-bench",
            limit=50,
            filters={"language": "python"},
        )
        assert ts.path == "/data/tasks.jsonl"
        assert ts.source_type == "swebench"
        assert ts.limit == 50
        assert ts.filters["language"] == "python"


class TestAgentConfig:
    """测试 Agent 配置."""

    def test_defaults(self):
        """默认值."""
        ac = AgentConfig()
        assert ac.framework == "openhands"
        assert ac.model == "claude-sonnet-4-20250514"
        assert ac.max_steps == 30
        assert ac.temperature == 0.0

    def test_custom(self):
        """自定义 Agent."""
        ac = AgentConfig(framework="sweagent", model="gpt-4o", max_steps=50)
        assert ac.framework == "sweagent"
        assert ac.model == "gpt-4o"
        assert ac.max_steps == 50


class TestSandboxConfig:
    """测试沙箱配置."""

    def test_defaults(self):
        """默认值."""
        sc = SandboxConfig()
        assert sc.image == "knowlyr/sandbox:latest"
        assert sc.timeout == 600
        assert sc.memory_limit == "4g"
        assert sc.network_enabled is False


class TestRewardConfig:
    """测试 Reward 配置."""

    def test_defaults(self):
        """默认值."""
        rc = RewardConfig()
        assert rc.enable_rule_layer is True
        assert rc.enable_model_layer is False
        assert "goal_progress" in rc.rubric_weights

    def test_custom_weights(self):
        """自定义权重."""
        rc = RewardConfig(rubric_weights={"custom": 1.0})
        assert rc.rubric_weights == {"custom": 1.0}


class TestPipelineConfig:
    """测试 Pipeline 全局配置."""

    def test_defaults(self):
        """默认配置."""
        pc = PipelineConfig()
        assert pc.output_dir == "./output"
        assert pc.parallel_workers == 1
        assert pc.checkpoint_interval == 10
        assert len(pc.agents) == 1

    def test_custom(self):
        """自定义 Pipeline."""
        pc = PipelineConfig(
            task_source=TaskSource(path="/tasks.jsonl"),
            agents=[
                AgentConfig(framework="openhands"),
                AgentConfig(framework="sweagent", model="gpt-4o"),
            ],
            output_dir="/data/output",
            parallel_workers=4,
        )
        assert pc.task_source.path == "/tasks.jsonl"
        assert len(pc.agents) == 2
        assert pc.agents[1].model == "gpt-4o"
        assert pc.parallel_workers == 4

    def test_nested_defaults(self):
        """嵌套配置的默认值应正确."""
        pc = PipelineConfig()
        assert pc.sandbox_config.timeout == 600
        assert pc.reward_config.enable_rule_layer is True
        assert pc.task_source.source_type == "jsonl"

    def test_model_dump(self):
        """应能序列化."""
        pc = PipelineConfig(output_dir="/test")
        d = pc.model_dump()
        assert d["output_dir"] == "/test"
        assert "agents" in d
        assert "sandbox_config" in d



"""CLI - 命令行界面."""

import json
import sys
from pathlib import Path
from typing import Optional

import click

from trajectoryhub import __version__
from trajectoryhub.config import AgentConfig, PipelineConfig, TaskSource
from trajectoryhub.exporter import DatasetExporter
from trajectoryhub.pipeline import Pipeline
from trajectoryhub.tasks import TaskLoader


@click.group()
@click.version_option(version=__version__, prog_name="knowlyr-hub")
def main():
    """AgentTrajectoryHub - Agent 轨迹数据 Pipeline 编排层

    串联 Sandbox -> Recorder -> Reward 全流程，产出可训练的数据集。
    """
    pass


@main.command()
@click.argument("task_source", type=click.Path(exists=True))
@click.option("-o", "--output", type=click.Path(), default="./output", help="输出目录 (默认: ./output)")
@click.option(
    "-f",
    "--framework",
    type=click.Choice(["openhands", "sweagent", "custom"]),
    default="openhands",
    help="Agent 框架",
)
@click.option("-m", "--model", type=str, default="claude-sonnet-4-20250514", help="LLM 模型")
@click.option("--max-steps", type=int, default=30, help="Agent 最大执行步数")
@click.option("-w", "--workers", type=int, default=1, help="并行工作进程数")
@click.option("--resume", type=click.Path(), default=None, help="从 checkpoint 恢复执行")
def run(
    task_source: str,
    output: str,
    framework: str,
    model: str,
    max_steps: int,
    workers: int,
    resume: Optional[str],
):
    """运行完整 Pipeline

    TASK_SOURCE: 任务定义文件路径 (JSONL 格式)
    """
    config = PipelineConfig(
        task_source=TaskSource(path=task_source),
        agents=[
            AgentConfig(
                framework=framework,
                model=model,
                max_steps=max_steps,
            )
        ],
        output_dir=output,
        parallel_workers=workers,
    )

    click.echo("启动 Agent 轨迹 Pipeline...")
    click.echo(f"  任务来源: {task_source}")
    click.echo(f"  Agent: {framework} ({model})")
    click.echo(f"  输出目录: {output}")

    pipeline = Pipeline(config)

    if resume:
        click.echo(f"  从 checkpoint 恢复: {resume}")
        result = pipeline.resume(resume)
    else:
        result = pipeline.run()

    if result.completed > 0 or result.failed > 0:
        click.echo("\nPipeline 执行完成:")
        click.echo(f"  总任务数: {result.total_tasks}")
        click.echo(f"  完成: {result.completed}")
        click.echo(f"  失败: {result.failed}")
        click.echo(f"  轨迹输出: {result.trajectories_path}")
        click.echo(f"  偏好对输出: {result.preferences_path}")
        click.echo(f"  质检报告: {result.quality_report_path}")
        click.echo(f"  耗时: {result.duration_seconds:.1f}s")
    else:
        click.echo("\n没有找到可执行的任务。请检查任务文件格式。")


@main.command()
@click.option(
    "--format",
    "export_format",
    type=click.Choice(["sft", "dpo", "benchmark", "huggingface"]),
    required=True,
    help="导出格式",
)
@click.option("-t", "--trajectories", type=click.Path(exists=True), required=True, help="轨迹文件路径")
@click.option("-p", "--preferences", type=click.Path(exists=True), default=None, help="偏好对文件路径")
@click.option("-o", "--output", type=click.Path(), required=True, help="输出文件路径")
@click.option("--repo-id", type=str, default=None, help="HuggingFace 仓库 ID (仅 huggingface 格式)")
def export(
    export_format: str,
    trajectories: str,
    preferences: Optional[str],
    output: str,
    repo_id: Optional[str],
):
    """导出数据集

    将轨迹数据导出为训练格式 (SFT / DPO / Benchmark)。
    """
    exporter = DatasetExporter(
        trajectories_dir=trajectories,
        preferences_dir=preferences,
    )

    click.echo(f"导出数据集 ({export_format})...")

    if export_format == "sft":
        result = exporter.export_sft(output)
    elif export_format == "dpo":
        result = exporter.export_dpo(output)
    elif export_format == "benchmark":
        result = exporter.export_benchmark(output)
    elif export_format == "huggingface":
        if not repo_id:
            click.echo("HuggingFace 导出需要指定 --repo-id", err=True)
            sys.exit(1)
        result = exporter.export_huggingface(repo_id)
    else:
        click.echo(f"不支持的格式: {export_format}", err=True)
        sys.exit(1)

    if result.success:
        click.echo("导出成功:")
        click.echo(f"  格式: {result.format}")
        click.echo(f"  输出: {result.output_path}")
        click.echo(f"  记录数: {result.total_records}")
    else:
        click.echo(f"导出失败: {result.error}", err=True)
        sys.exit(1)


@main.command()
@click.argument("log_path", type=click.Path(exists=True))
@click.option(
    "-f",
    "--framework",
    type=click.Choice(["openhands", "sweagent", "swe-agent"]),
    required=True,
    help="Agent 框架",
)
@click.option("-o", "--output", type=click.Path(), default="./output", help="输出目录 (默认: ./output)")
@click.option("--save", is_flag=True, help="保存轨迹到输出目录的 trajectories.jsonl")
def process(log_path: str, framework: str, output: str, save: bool):
    """处理单个 Agent 日志文件

    LOG_PATH: Agent 日志文件路径 (OpenHands JSONL / SWE-agent JSON)
    """
    config = PipelineConfig(output_dir=output)
    pipeline = Pipeline(config)

    click.echo(f"处理日志: {log_path} ({framework})...")

    try:
        traj = pipeline.run_from_log(log_path, framework)
    except (RuntimeError, ValueError) as e:
        click.echo(f"处理失败: {e}", err=True)
        sys.exit(1)

    click.echo("处理完成:")
    click.echo(f"  任务 ID: {traj.task_id}")
    click.echo(f"  框架: {traj.agent_framework}")
    click.echo(f"  模型: {traj.agent_model}")
    click.echo(f"  步数: {traj.total_steps}")
    click.echo(f"  成功: {traj.success}")
    click.echo(f"  Reward: {traj.reward:.3f}")
    click.echo(f"  耗时: {traj.duration_seconds:.1f}s")

    if save:
        output_dir = Path(output)
        output_dir.mkdir(parents=True, exist_ok=True)
        traj_path = output_dir / "trajectories.jsonl"
        with open(traj_path, "a", encoding="utf-8") as f:
            f.write(json.dumps({
                "task_id": traj.task_id,
                "agent_framework": traj.agent_framework,
                "agent_model": traj.agent_model,
                "steps": traj.steps,
                "total_steps": traj.total_steps,
                "success": traj.success,
                "reward": traj.reward,
                "step_rewards": traj.step_rewards,
                "duration_seconds": traj.duration_seconds,
                "metadata": traj.metadata,
            }, ensure_ascii=False) + "\n")
        click.echo(f"  已保存: {traj_path}")


@main.command("process-batch")
@click.argument("log_dir", type=click.Path(exists=True))
@click.option(
    "-f",
    "--framework",
    type=click.Choice(["openhands", "sweagent", "swe-agent"]),
    required=True,
    help="Agent 框架",
)
@click.option("-o", "--output", type=click.Path(), default="./output", help="输出目录 (默认: ./output)")
@click.option("-p", "--pattern", type=str, default="*", help="文件匹配模式 (默认: *)")
def process_batch(log_dir: str, framework: str, output: str, pattern: str):
    """批量处理 Agent 日志目录

    LOG_DIR: 包含日志文件的目录
    """
    config = PipelineConfig(output_dir=output)
    pipeline = Pipeline(config)

    click.echo(f"批量处理日志: {log_dir} ({framework}, pattern={pattern})...")

    try:
        trajectories = pipeline.run_batch_from_logs(log_dir, framework, pattern)
    except (RuntimeError, ValueError) as e:
        click.echo(f"处理失败: {e}", err=True)
        sys.exit(1)

    if not trajectories:
        click.echo("没有找到匹配的日志文件。")
        return

    # 保存轨迹
    output_dir = Path(output)
    output_dir.mkdir(parents=True, exist_ok=True)
    traj_path = output_dir / "trajectories.jsonl"
    with open(traj_path, "w", encoding="utf-8") as f:
        for traj in trajectories:
            f.write(json.dumps({
                "task_id": traj.task_id,
                "agent_framework": traj.agent_framework,
                "agent_model": traj.agent_model,
                "steps": traj.steps,
                "total_steps": traj.total_steps,
                "success": traj.success,
                "reward": traj.reward,
                "step_rewards": traj.step_rewards,
                "duration_seconds": traj.duration_seconds,
                "metadata": traj.metadata,
            }, ensure_ascii=False) + "\n")

    # 统计
    success_count = sum(1 for t in trajectories if t.success)
    avg_reward = sum(t.reward for t in trajectories) / len(trajectories) if trajectories else 0.0

    click.echo("批量处理完成:")
    click.echo(f"  轨迹数: {len(trajectories)}")
    click.echo(f"  成功率: {success_count}/{len(trajectories)}")
    click.echo(f"  平均 Reward: {avg_reward:.3f}")
    click.echo(f"  输出: {traj_path}")


@main.command()
@click.argument("output_dir", type=click.Path(exists=True))
def status(output_dir: str):
    """查看 Pipeline 状态

    OUTPUT_DIR: Pipeline 输出目录
    """
    output = Path(output_dir)

    click.echo(f"Pipeline 状态 ({output_dir}):")

    # Checkpoint
    checkpoint_path = output / "checkpoint.json"
    if checkpoint_path.exists():
        with open(checkpoint_path, "r", encoding="utf-8") as f:
            checkpoint = json.load(f)
        click.echo(
            f"  Checkpoint: 已完成 {checkpoint.get('completed', 0)} 个, "
            f"失败 {checkpoint.get('failed', 0)} 个"
        )
    else:
        click.echo("  Checkpoint: 无")

    # Trajectories
    trajectories_path = output / "trajectories.jsonl"
    if trajectories_path.exists():
        with open(trajectories_path, "r", encoding="utf-8") as f:
            lines = [line for line in f if line.strip()]
        click.echo(f"  轨迹文件: {len(lines)} 条")

        # 统计成功率和平均 reward
        rewards = []
        success_count = 0
        for line in lines:
            data = json.loads(line)
            if data.get("success"):
                success_count += 1
            rewards.append(data.get("reward", 0.0))

        if rewards:
            avg_reward = sum(rewards) / len(rewards)
            click.echo(f"  成功率: {success_count}/{len(lines)} ({success_count/len(lines)*100:.1f}%)")
            click.echo(f"  平均 Reward: {avg_reward:.3f}")
    else:
        click.echo("  轨迹文件: 未生成")

    # Preferences
    preferences_path = output / "preferences.jsonl"
    if preferences_path.exists():
        with open(preferences_path, "r", encoding="utf-8") as f:
            count = sum(1 for line in f if line.strip())
        click.echo(f"  偏好对: {count} 对")
    else:
        click.echo("  偏好对: 未生成")

    # Quality report
    report_path = output / "quality_report.json"
    if report_path.exists():
        click.echo("  质检报告: 已生成")
    else:
        click.echo("  质检报告: 未生成")


@main.command()
@click.argument("source", type=click.Path(exists=True))
@click.option("--language", type=str, default=None, help="按语言过滤")
@click.option("--difficulty", type=click.Choice(["easy", "medium", "hard"]), default=None, help="按难度过滤")
@click.option("--type", "task_type", type=str, default=None, help="按类型过滤")
@click.option("--limit", type=int, default=None, help="限制显示数量")
def tasks(
    source: str,
    language: Optional[str],
    difficulty: Optional[str],
    task_type: Optional[str],
    limit: Optional[int],
):
    """列出和过滤任务

    SOURCE: 任务定义文件路径 (JSONL 格式)
    """
    loader = TaskLoader()
    all_tasks = loader.load_from_jsonl(source)

    # 过滤
    filtered = loader.filter_tasks(
        all_tasks,
        language=language,
        difficulty=difficulty,
        task_type=task_type,
    )

    if limit:
        filtered = filtered[:limit]

    click.echo(f"任务列表 (共 {len(filtered)} 个，总计 {len(all_tasks)} 个):")
    click.echo("")

    for task in filtered:
        click.echo(f"  [{task.task_id}]")
        click.echo(f"    类型: {task.type} | 语言: {task.language} | 难度: {task.difficulty}")
        if task.description:
            desc = task.description[:80] + "..." if len(task.description) > 80 else task.description
            click.echo(f"    描述: {desc}")
        if task.repo:
            click.echo(f"    仓库: {task.repo}")
        click.echo("")


@main.command()
@click.option("-t", "--trajectories", type=click.Path(exists=True), required=True, help="轨迹文件路径")
@click.option("-p", "--preferences", type=click.Path(exists=True), default=None, help="偏好对文件路径")
@click.option("--repo-id", type=str, required=True, help="HuggingFace 仓库 ID")
@click.option("--generate-card", is_flag=True, help="同时生成 Dataset Card")
def publish(
    trajectories: str,
    preferences: Optional[str],
    repo_id: str,
    generate_card: bool,
):
    """发布数据集到 HuggingFace

    将轨迹和偏好对数据推送到 HuggingFace Hub。
    """
    exporter = DatasetExporter(
        trajectories_dir=trajectories,
        preferences_dir=preferences,
    )

    if generate_card:
        card = exporter.generate_datacard()
        click.echo("Dataset Card:")
        click.echo(card)
        click.echo("---")

    click.echo(f"发布到 HuggingFace: {repo_id}...")
    result = exporter.export_huggingface(repo_id)

    if result.success:
        click.echo(f"发布成功: https://huggingface.co/datasets/{repo_id}")
    else:
        click.echo(f"发布失败: {result.error}", err=True)
        sys.exit(1)


if __name__ == "__main__":
    main()



"""测试 collect() — Gymnasium 风格轨迹收集."""

from knowlyrcore.env import AgentEnv
from knowlyrcore.timestep import TimeStep

from trajectoryhub.collect import collect


# ── MockEnv ─────────────────────────────────────────────────────────


class MockEnv(AgentEnv):
    """测试用环境."""

    domain = "test"

    def __init__(self):
        self._step_count = 0
        self._closed = False

    def reset(self, *, task=None, seed=None) -> TimeStep:
        self._step_count = 0
        return TimeStep(observation="ready", info={"task": task})

    def step(self, action: dict) -> TimeStep:
        self._step_count += 1
        tool = action.get("tool", "noop")
        return TimeStep(
            observation=f"{tool}-result",
            terminated=(tool == "submit"),
            info={"exit_code": 0, "step": self._step_count},
        )

    def close(self):
        self._closed = True

    @property
    def available_tools(self):
        return ["bash", "submit"]


# ── mock agent ──────────────────────────────────────────────────────


def simple_agent(observation: str) -> dict:
    """3 步 bash 后 submit 的简单 agent."""
    if "step-3" in observation or observation.count("bash") >= 3:
        return {"tool": "submit", "params": {}}
    return {"tool": "bash", "params": {"command": "ls"}}


class StatefulAgent:
    """有状态的 agent，第 n 步后 submit."""

    def __init__(self, submit_after: int = 3):
        self._step = 0
        self._submit_after = submit_after

    def __call__(self, observation: str) -> dict:
        self._step += 1
        if self._step >= self._submit_after:
            self._step = 0  # 重置，支持多 episode
            return {"tool": "submit", "params": {}}
        return {"tool": "bash", "params": {"command": f"step-{self._step}"}}


# ── collect 测试 ──────────────────────────────────────────────────


class TestCollect:
    """collect() 测试."""

    def test_single_episode(self):
        """单 episode 收集."""
        agent = StatefulAgent(submit_after=3)
        trajs = collect(MockEnv(), agent=agent, n_episodes=1, max_steps=10)

        assert len(trajs) == 1
        traj = trajs[0]
        assert len(traj["steps"]) == 3
        assert traj["outcome"]["success"] is True
        assert traj["outcome"]["terminated"] is True

    def test_multiple_episodes(self):
        """多 episode 收集."""
        agent = StatefulAgent(submit_after=2)
        trajs = collect(MockEnv(), agent=agent, n_episodes=3, max_steps=10)

        assert len(trajs) == 3
        for traj in trajs:
            assert len(traj["steps"]) == 2
            assert traj["outcome"]["success"] is True

    def test_max_steps_truncation(self):
        """超过 max_steps 应 truncate."""
        def never_submit(obs):
            return {"tool": "bash", "params": {}}

        trajs = collect(MockEnv(), agent=never_submit, n_episodes=1, max_steps=5)

        assert len(trajs) == 1
        traj = trajs[0]
        assert len(traj["steps"]) == 5
        assert traj["outcome"]["truncated"] is True

    def test_agent_name_model_name(self):
        """agent_name 和 model_name 应记录在轨迹中."""
        agent = StatefulAgent(submit_after=1)
        trajs = collect(
            MockEnv(),
            agent=agent,
            n_episodes=1,
            agent_name="test-agent",
            model_name="gpt-4o",
        )

        traj = trajs[0]
        assert traj["agent"] == "test-agent"
        assert traj["model"] == "gpt-4o"

    def test_task_passed_to_env(self):
        """task 应传给 env.reset()."""
        class FakeTask:
            task_id = "task-001"
            description = "Fix the bug"

        agent = StatefulAgent(submit_after=1)
        trajs = collect(
            MockEnv(),
            agent=agent,
            n_episodes=1,
            task=FakeTask(),
        )

        traj = trajs[0]
        assert traj["task"]["task_id"] == "task-001"

    def test_trajectory_has_steps_detail(self):
        """每步应有 tool/params/output 等详情."""
        agent = StatefulAgent(submit_after=2)
        trajs = collect(MockEnv(), agent=agent, n_episodes=1, max_steps=10)

        step = trajs[0]["steps"][0]
        assert "tool" in step
        assert "params" in step
        assert "output" in step
        assert "step_id" in step

    def test_domain_in_metadata(self):
        """metadata 应包含 domain."""
        agent = StatefulAgent(submit_after=1)
        trajs = collect(MockEnv(), agent=agent, n_episodes=1)

        assert trajs[0]["metadata"]["domain"] == "test"

    def test_env_string_id(self):
        """传入字符串 env_id 应尝试 make()."""
        from knowlyrcore.registry import register, _clear_registry

        _clear_registry()
        register("test/mock", MockEnv, domain="test")

        agent = StatefulAgent(submit_after=1)
        trajs = collect("test/mock", agent=agent, n_episodes=1)

        assert len(trajs) == 1
        _clear_registry()



"""测试 CLI process / process-batch 命令."""

import json

from click.testing import CliRunner

from trajectoryhub.cli import main


def _write_openhands_log(path):
    """写最小的 OpenHands JSONL 日志."""
    events = [
        {
            "id": 0, "source": "user", "action": "message",
            "args": {"content": "修复 bug", "task_id": "cli-test"},
            "timestamp": "2026-01-01T10:00:00Z",
        },
        {
            "id": 1, "source": "agent", "action": "run",
            "args": {"command": "pytest"},
            "thought": "跑测试",
            "timestamp": "2026-01-01T10:01:00Z",
            "extras": {"model": "claude-sonnet-4-20250514", "agent_class": "CodeActAgent"},
        },
        {
            "id": 2, "observation": "run",
            "content": "1 passed",
            "extras": {"exit_code": 0},
        },
        {
            "id": 3, "action": "finish",
            "args": {"outputs": {"test_result": {
                "success": True, "tests_passed": 1, "tests_failed": 0,
            }}},
        },
    ]
    with open(path, "w") as f:
        for e in events:
            f.write(json.dumps(e) + "\n")


def _write_sweagent_log(path, task_desc="修复 Django bug"):
    """写最小的 SWE-agent JSON 日志."""
    data = {
        "problem_statement": task_desc,
        "history": [
            [{"action": "bash", "command": "ls", "thought": "看看文件"}, "file.py\n"],
            [{"action": "submit"}, "Submitted"],
        ],
        "info": {
            "instance_id": "swe-test-001",
            "model_name": "gpt-4o",
            "repo": "django/django",
            "base_commit": "abc123",
            "exit_status": "submitted",
        },
    }
    with open(path, "w") as f:
        json.dump(data, f)


class TestProcessCommand:
    """测试 process 命令."""

    def test_process_openhands(self, tmp_path):
        """处理 OpenHands 日志."""
        log_path = tmp_path / "log.jsonl"
        _write_openhands_log(log_path)

        runner = CliRunner()
        result = runner.invoke(main, [
            "process", str(log_path),
            "-f", "openhands",
            "-o", str(tmp_path / "out"),
        ])

        assert result.exit_code == 0
        assert "处理完成" in result.output
        assert "cli-test" in result.output
        assert "Reward" in result.output

    def test_process_sweagent(self, tmp_path):
        """处理 SWE-agent 日志."""
        log_path = tmp_path / "traj.json"
        _write_sweagent_log(log_path)

        runner = CliRunner()
        result = runner.invoke(main, [
            "process", str(log_path),
            "-f", "sweagent",
            "-o", str(tmp_path / "out"),
        ])

        assert result.exit_code == 0
        assert "处理完成" in result.output
        assert "swe-test-001" in result.output

    def test_process_with_save(self, tmp_path):
        """使用 --save 应写入 trajectories.jsonl."""
        log_path = tmp_path / "log.jsonl"
        _write_openhands_log(log_path)
        out_dir = tmp_path / "out"

        runner = CliRunner()
        result = runner.invoke(main, [
            "process", str(log_path),
            "-f", "openhands",
            "-o", str(out_dir),
            "--save",
        ])

        assert result.exit_code == 0
        assert "已保存" in result.output

        traj_file = out_dir / "trajectories.jsonl"
        assert traj_file.exists()

        with open(traj_file) as f:
            records = [json.loads(line) for line in f if line.strip()]
        assert len(records) == 1
        assert records[0]["task_id"] == "cli-test"


class TestProcessBatchCommand:
    """测试 process-batch 命令."""

    def test_batch_sweagent(self, tmp_path):
        """批量处理 SWE-agent 日志."""
        log_dir = tmp_path / "logs"
        log_dir.mkdir()
        for i in range(3):
            _write_sweagent_log(log_dir / f"traj_{i}.json", f"任务 {i}")

        out_dir = tmp_path / "out"

        runner = CliRunner()
        result = runner.invoke(main, [
            "process-batch", str(log_dir),
            "-f", "sweagent",
            "-o", str(out_dir),
            "-p", "*.json",
        ])

        assert result.exit_code == 0
        assert "批量处理完成" in result.output
        assert "轨迹数: 3" in result.output

        traj_file = out_dir / "trajectories.jsonl"
        assert traj_file.exists()

    def test_batch_empty_dir(self, tmp_path):
        """空目录应提示无文件."""
        log_dir = tmp_path / "empty"
        log_dir.mkdir()

        runner = CliRunner()
        result = runner.invoke(main, [
            "process-batch", str(log_dir),
            "-f", "sweagent",
            "-o", str(tmp_path / "out"),
        ])

        assert result.exit_code == 0
        assert "没有找到" in result.output



"""Dataset export - 数据集导出.

支持将轨迹数据导出为多种训练格式：
- SFT: 监督微调格式 (instruction/response 对)
- DPO: 偏好学习格式 (chosen/rejected 对)
- Benchmark: 评测基准格式
- HuggingFace: 推送到 HuggingFace Hub
"""

import json
import logging
import tempfile
from dataclasses import dataclass
from pathlib import Path
from typing import Any, Dict, List, Optional

logger = logging.getLogger(__name__)

try:
    from huggingface_hub import HfApi

    _HAS_HF = True
except ImportError:
    _HAS_HF = False


@dataclass
class ExportResult:
    """导出结果.

    Attributes:
        success: 是否成功
        output_path: 输出文件路径
        total_records: 导出记录数
        format: 导出格式
        error: 错误信息 (如果失败)
    """

    success: bool = False
    output_path: str = ""
    total_records: int = 0
    format: str = ""
    error: Optional[str] = None


class DatasetExporter:
    """数据集导出器.

    将 Pipeline 产出的轨迹和偏好对导出为各种训练格式。

    Usage:
        exporter = DatasetExporter(
            trajectories_dir="./output/trajectories.jsonl",
            preferences_dir="./output/preferences.jsonl",
        )

        # 导出 SFT 格式
        result = exporter.export_sft("./export/sft_train.jsonl")

        # 导出 DPO 格式
        result = exporter.export_dpo("./export/dpo_train.jsonl")
    """

    def __init__(
        self,
        trajectories_dir: str,
        preferences_dir: Optional[str] = None,
    ) -> None:
        self.trajectories_dir = Path(trajectories_dir)
        self.preferences_dir = Path(preferences_dir) if preferences_dir else None

    def export_sft(self, output_path: str) -> ExportResult:
        """导出为 SFT 训练格式.

        将成功的轨迹转换为 instruction/response 对，适用于监督微调。

        SFT 格式 (每行一个 JSON):
        ```json
        {
            "instruction": "任务描述",
            "input": "任务上下文 (repo, commit 等)",
            "response": "Agent 执行步骤序列",
            "task_id": "原始任务 ID",
            "reward": 0.85,
            "metadata": {...}
        }
        ```

        只导出成功的轨迹 (success=True)，按 reward 从高到低排序。

        Args:
            output_path: 输出文件路径

        Returns:
            ExportResult: 导出结果
        """
        try:
            trajectories = self._load_trajectories()
            logger.info("导出 SFT: 加载 %d 条轨迹", len(trajectories))
            # 只选成功的轨迹，按 reward 排序
            successful = sorted(
                [t for t in trajectories if t.get("success", False)],
                key=lambda t: t.get("reward", 0.0),
                reverse=True,
            )

            output = Path(output_path)
            output.parent.mkdir(parents=True, exist_ok=True)

            with open(output, "w", encoding="utf-8") as f:
                for traj in successful:
                    # 将步骤序列转为 response 文本
                    response_parts = []
                    for i, step in enumerate(traj.get("steps", []), 1):
                        action = step.get("action", "")
                        observation = step.get("observation", "")
                        thought = step.get("thought", "")

                        step_text = f"Step {i}:"
                        if thought:
                            step_text += f"\nThought: {thought}"
                        if action:
                            step_text += f"\nAction: {action}"
                        if observation:
                            step_text += f"\nObservation: {observation}"
                        response_parts.append(step_text)

                    metadata = traj.get("metadata", {})
                    record = {
                        "instruction": metadata.get(
                            "task_description", f"Solve task: {traj.get('task_id', '')}"
                        ),
                        "input": json.dumps(
                            {
                                "repo": metadata.get("repo", ""),
                                "base_commit": metadata.get("base_commit", ""),
                                "test_command": metadata.get("test_command", ""),
                            },
                            ensure_ascii=False,
                        ),
                        "response": "\n\n".join(response_parts) if response_parts else "",
                        "task_id": traj.get("task_id", ""),
                        "reward": traj.get("reward", 0.0),
                        "metadata": {
                            "agent_framework": traj.get("agent_framework", ""),
                            "agent_model": traj.get("agent_model", ""),
                            "total_steps": traj.get("total_steps", 0),
                        },
                    }
                    f.write(json.dumps(record, ensure_ascii=False) + "\n")

            logger.info("SFT 导出完成: %d 条记录 -> %s", len(successful), output)
            return ExportResult(
                success=True,
                output_path=str(output),
                total_records=len(successful),
                format="sft",
            )

        except Exception as e:
            logger.exception("SFT 导出失败")
            return ExportResult(
                success=False,
                format="sft",
                error=str(e),
            )

    def export_dpo(self, output_path: str) -> ExportResult:
        """导出为 DPO 训练格式.

        将偏好对转换为 DPO 训练格式。

        DPO 格式 (每行一个 JSON):
        ```json
        {
            "prompt": "任务描述",
            "chosen": "高 reward 的执行轨迹",
            "rejected": "低 reward 的执行轨迹",
            "task_id": "原始任务 ID",
            "reward_margin": 0.3,
            "metadata": {...}
        }
        ```

        Args:
            output_path: 输出文件路径

        Returns:
            ExportResult: 导出结果
        """
        if not self.preferences_dir or not self.preferences_dir.exists():
            return ExportResult(
                success=False,
                format="dpo",
                error="偏好对文件不存在，请先运行 Pipeline 生成偏好对",
            )

        try:
            preferences = self._load_preferences()
            logger.info("导出 DPO: 加载 %d 个偏好对", len(preferences))

            output = Path(output_path)
            output.parent.mkdir(parents=True, exist_ok=True)

            with open(output, "w", encoding="utf-8") as f:
                for pref in preferences:
                    # 将步骤序列转为文本
                    chosen_text = self._steps_to_text(pref.get("chosen", {}).get("steps", []))
                    rejected_text = self._steps_to_text(
                        pref.get("rejected", {}).get("steps", [])
                    )

                    record = {
                        "prompt": f"Solve the following task:\n\nTask ID: {pref.get('task_id', '')}",
                        "chosen": chosen_text,
                        "rejected": rejected_text,
                        "task_id": pref.get("task_id", ""),
                        "reward_margin": pref.get("reward_margin", 0.0),
                        "metadata": {
                            "chosen_model": pref.get("chosen", {}).get("agent_model", ""),
                            "rejected_model": pref.get("rejected", {}).get("agent_model", ""),
                            "chosen_reward": pref.get("chosen", {}).get("reward", 0.0),
                            "rejected_reward": pref.get("rejected", {}).get("reward", 0.0),
                        },
                    }
                    f.write(json.dumps(record, ensure_ascii=False) + "\n")

            logger.info("DPO 导出完成: %d 条记录 -> %s", len(preferences), output)
            return ExportResult(
                success=True,
                output_path=str(output),
                total_records=len(preferences),
                format="dpo",
            )

        except Exception as e:
            logger.exception("DPO 导出失败")
            return ExportResult(
                success=False,
                format="dpo",
                error=str(e),
            )

    def export_benchmark(self, output_path: str) -> ExportResult:
        """导出为评测基准格式.

        将任务和轨迹导出为可复现的评测基准格式。

        Benchmark 格式:
        ```json
        {
            "task_id": "...",
            "description": "...",
            "repo": "...",
            "base_commit": "...",
            "test_command": "...",
            "reference_trajectories": [...],
            "difficulty": "medium",
            "expected_reward_range": [0.6, 0.9]
        }
        ```

        Args:
            output_path: 输出文件路径

        Returns:
            ExportResult: 导出结果
        """
        try:
            trajectories = self._load_trajectories()
            logger.info("导出 Benchmark: 加载 %d 条轨迹", len(trajectories))

            # 按 task_id 分组
            task_groups: Dict[str, List[Dict[str, Any]]] = {}
            for traj in trajectories:
                tid = traj.get("task_id", "")
                if tid not in task_groups:
                    task_groups[tid] = []
                task_groups[tid].append(traj)

            output = Path(output_path)
            output.parent.mkdir(parents=True, exist_ok=True)

            records = []
            with open(output, "w", encoding="utf-8") as f:
                for task_id, group in task_groups.items():
                    rewards = [t.get("reward", 0.0) for t in group]
                    metadata = group[0].get("metadata", {})

                    record = {
                        "task_id": task_id,
                        "description": metadata.get("task_description", ""),
                        "repo": metadata.get("repo", ""),
                        "base_commit": metadata.get("base_commit", ""),
                        "test_command": metadata.get("test_command", ""),
                        "reference_trajectories": [
                            {
                                "agent_model": t.get("agent_model", ""),
                                "reward": t.get("reward", 0.0),
                                "success": t.get("success", False),
                                "total_steps": t.get("total_steps", 0),
                            }
                            for t in group
                        ],
                        "difficulty": metadata.get("difficulty", "unknown"),
                        "expected_reward_range": [
                            round(min(rewards), 2) if rewards else 0.0,
                            round(max(rewards), 2) if rewards else 0.0,
                        ],
                    }
                    f.write(json.dumps(record, ensure_ascii=False) + "\n")
                    records.append(record)

            logger.info("Benchmark 导出完成: %d 条记录 -> %s", len(records), output)
            return ExportResult(
                success=True,
                output_path=str(output),
                total_records=len(records),
                format="benchmark",
            )

        except Exception as e:
            logger.exception("Benchmark 导出失败")
            return ExportResult(
                success=False,
                format="benchmark",
                error=str(e),
            )

    def export_huggingface(self, repo_id: str) -> ExportResult:
        """推送数据集到 HuggingFace Hub.

        将轨迹和偏好对打包上传到 HuggingFace，附带自动生成的 Dataset Card。
        需要安装 huggingface-hub 并已通过 huggingface-cli login 认证。

        上传文件:
        - README.md (Dataset Card)
        - sft_train.jsonl (SFT 格式)
        - dpo_train.jsonl (DPO 格式，如有偏好对)
        - benchmark.jsonl (评测基准格式)

        Args:
            repo_id: HuggingFace 仓库 ID (e.g. "username/dataset-name")

        Returns:
            ExportResult: 导出结果
        """
        if not _HAS_HF:
            return ExportResult(
                success=False,
                format="huggingface",
                error="需要安装 huggingface-hub: pip install knowlyr-hub[hf]",
            )

        try:
            api = HfApi()

            # 创建 dataset repo（已存在则跳过）
            api.create_repo(repo_id, repo_type="dataset", exist_ok=True)
            logger.info("HuggingFace repo 已就绪: %s", repo_id)

            uploaded_count = 0

            with tempfile.TemporaryDirectory() as tmp_dir:
                tmp_path = Path(tmp_dir)

                # 生成 Dataset Card
                card = self.generate_datacard()
                readme_path = tmp_path / "README.md"
                readme_path.write_text(card, encoding="utf-8")

                # 导出 SFT
                sft_path = tmp_path / "sft_train.jsonl"
                sft_result = self.export_sft(str(sft_path))
                if sft_result.success:
                    uploaded_count += sft_result.total_records

                # 导出 DPO（如有偏好对）
                if self.preferences_dir and self.preferences_dir.exists():
                    dpo_path = tmp_path / "dpo_train.jsonl"
                    dpo_result = self.export_dpo(str(dpo_path))
                    if dpo_result.success:
                        uploaded_count += dpo_result.total_records

                # 导出 Benchmark
                bench_path = tmp_path / "benchmark.jsonl"
                bench_result = self.export_benchmark(str(bench_path))
                if bench_result.success:
                    uploaded_count += bench_result.total_records

                # 上传整个目录
                api.upload_folder(
                    repo_id=repo_id,
                    folder_path=str(tmp_path),
                    repo_type="dataset",
                )

            logger.info(
                "HuggingFace 上传完成: %s (%d 条记录)", repo_id, uploaded_count,
            )
            return ExportResult(
                success=True,
                output_path=f"https://huggingface.co/datasets/{repo_id}",
                total_records=uploaded_count,
                format="huggingface",
            )

        except Exception as e:
            logger.exception("HuggingFace 导出失败")
            return ExportResult(
                success=False,
                format="huggingface",
                error=str(e),
            )

    def generate_datacard(self) -> str:
        """生成数据集卡片 (Dataset Card).

        Returns:
            str: Markdown 格式的数据集说明
        """
        trajectories = self._load_trajectories()

        total = len(trajectories)
        successful = sum(1 for t in trajectories if t.get("success", False))
        avg_reward = (
            sum(t.get("reward", 0.0) for t in trajectories) / total if total > 0 else 0.0
        )
        avg_steps = (
            sum(t.get("total_steps", 0) for t in trajectories) / total if total > 0 else 0.0
        )

        models = set(t.get("agent_model", "") for t in trajectories)
        frameworks = set(t.get("agent_framework", "") for t in trajectories)

        card = f"""---
language:
- en
- zh
tags:
- agent-trajectory
- code-agent
- sft
- dpo
- reinforcement-learning
license: mit
---

# Agent Trajectory Dataset

## Dataset Description

Agent 执行轨迹数据集，包含 Code Agent 在真实代码仓库上的执行过程记录。
每条轨迹包含完整的步骤序列、过程级 Reward 打分和偏好对。

## Dataset Statistics

| Metric | Value |
|--------|-------|
| Total trajectories | {total} |
| Successful | {successful} |
| Success rate | {successful / total * 100:.1f}% |
| Average reward | {avg_reward:.3f} |
| Average steps | {avg_steps:.1f} |
| Agent models | {', '.join(models)} |
| Agent frameworks | {', '.join(frameworks)} |

## Data Format

### Trajectories (SFT)

Each record contains:
- `instruction`: Task description
- `input`: Task context (repo, commit, test command)
- `response`: Agent execution steps
- `reward`: Process-level reward score

### Preferences (DPO)

Each record contains:
- `prompt`: Task description
- `chosen`: Higher-reward trajectory
- `rejected`: Lower-reward trajectory
- `reward_margin`: Reward difference

## Usage

```python
from datasets import load_dataset
dataset = load_dataset("path/to/dataset")
```

## License

MIT

## Citation

If you use this dataset, please cite:

```bibtex
@misc{{agent-trajectory-hub,
    title={{Agent Trajectory Hub}},
    author={{Liu Kai}},
    year={{2026}},
    url={{https://github.com/liuxiaotong/agent-trajectory-hub}}
}}
```
"""
        return card

    # ------------------------------------------------------------------
    # 内部方法
    # ------------------------------------------------------------------

    def _load_trajectories(self) -> List[Dict[str, Any]]:
        """从 JSONL 文件加载轨迹."""
        records = []
        if not self.trajectories_dir.exists():
            return records

        with open(self.trajectories_dir, "r", encoding="utf-8") as f:
            for line in f:
                line = line.strip()
                if line:
                    records.append(json.loads(line))
        return records

    def _load_preferences(self) -> List[Dict[str, Any]]:
        """加载偏好对，兼容 JSON 数组和 JSONL 两种格式."""
        records = []
        if not self.preferences_dir or not self.preferences_dir.exists():
            return records

        with open(self.preferences_dir, "r", encoding="utf-8") as f:
            content = f.read().strip()

        if not content:
            return records

        # 尝试 JSON 数组格式（knowlyr-reward preferences 输出）
        if content.startswith("["):
            data = json.loads(content)
            if isinstance(data, list):
                return data

        # 回退到 JSONL 格式（Pipeline 内部生成）
        for line in content.splitlines():
            line = line.strip()
            if line:
                records.append(json.loads(line))
        return records

    def _steps_to_text(self, steps: List[Dict[str, Any]]) -> str:
        """将步骤列表转为文本格式."""
        parts = []
        for i, step in enumerate(steps, 1):
            action = step.get("action", "")
            observation = step.get("observation", "")
            thought = step.get("thought", "")

            step_text = f"Step {i}:"
            if thought:
                step_text += f"\nThought: {thought}"
            if action:
                step_text += f"\nAction: {action}"
            if observation:
                step_text += f"\nObservation: {observation}"
            parts.append(step_text)

        return "\n\n".join(parts) if parts else ""



"""轨迹收集 — 在环境中运行 agent 并收集轨迹.

借鉴 AgentGym 的轨迹收集 + SWE-Gym 的训练数据生产模式。

Usage::

    from trajectoryhub.collect import collect

    def my_agent(observation: str) -> dict:
        return {"tool": "bash", "params": {"command": "ls"}}

    trajectories = collect("knowlyr/sandbox", agent=my_agent, n_episodes=5)
"""

from __future__ import annotations

import logging
from typing import Any, Callable

logger = logging.getLogger(__name__)

try:
    from knowlyrcore.registry import make as _make_env
    from knowlyrcore.wrappers import MaxStepsWrapper, RecorderWrapper

    _HAS_CORE_ENV = True
except ImportError:
    _HAS_CORE_ENV = False


def collect(
    env: Any,
    agent: Callable[[str], dict[str, Any]],
    n_episodes: int = 1,
    max_steps: int = 30,
    agent_name: str = "",
    model_name: str = "",
    task: Any = None,
) -> list[dict[str, Any]]:
    """在环境中运行 agent，收集轨迹.

    Args:
        env: AgentEnv 实例或环境 ID 字符串 (如 "knowlyr/sandbox")
        agent: agent 函数，签名 (observation: str) -> action dict
        n_episodes: 收集轮数
        max_steps: 每轮最大步数
        agent_name: Agent 名称（记录在轨迹中）
        model_name: 模型名称（记录在轨迹中）
        task: 任务信息（传给 env.reset()）

    Returns:
        轨迹字典列表，每个包含 task/agent/model/steps/outcome/metadata

    Raises:
        RuntimeError: knowlyr-core 未安装
    """
    if not _HAS_CORE_ENV:
        raise RuntimeError(
            "轨迹收集需要 knowlyr-core >= 0.1.0: pip install knowlyr-core"
        )

    # 解析 env：字符串 → make()，实例 → 直接用
    if isinstance(env, str):
        env = _make_env(env)

    # 包装：MaxSteps + Recorder
    env = MaxStepsWrapper(env, max_steps=max_steps)
    env = RecorderWrapper(env, agent_name=agent_name, model_name=model_name)

    trajectories: list[dict[str, Any]] = []

    for episode in range(n_episodes):
        ts = env.reset(task=task)
        logger.info("Episode %d/%d 开始", episode + 1, n_episodes)

        while not ts.done:
            action = agent(ts.observation)
            ts = env.step(action)

        traj = env.get_trajectory()
        trajectories.append(traj)
        logger.info(
            "Episode %d/%d 完成: %d 步, success=%s",
            episode + 1,
            n_episodes,
            len(traj["steps"]),
            traj["outcome"].get("success"),
        )

    env.close()
    logger.info("收集完成: %d 条轨迹", len(trajectories))
    return trajectories



"""Tests for pipeline configuration and basic setup."""

from trajectoryhub.config import (
    AgentConfig,
    PipelineConfig,
    RewardConfig,
    SandboxConfig,
    TaskSource,
)
from trajectoryhub.pipeline import Pipeline, PipelineResult, Trajectory
from trajectoryhub.tasks import TaskInfo


class TestPipelineConfig:
    """Tests for PipelineConfig."""

    def test_default_config(self):
        """Test default configuration values."""
        config = PipelineConfig()
        assert config.output_dir == "./output"
        assert config.parallel_workers == 1
        assert config.checkpoint_interval == 10
        assert len(config.agents) == 1

    def test_custom_config(self):
        """Test custom configuration."""
        config = PipelineConfig(
            task_source=TaskSource(path="tasks.jsonl"),
            agents=[
                AgentConfig(framework="openhands", model="claude-sonnet-4-20250514"),
                AgentConfig(framework="sweagent", model="gpt-4o"),
            ],
            output_dir="./custom_output",
            parallel_workers=4,
        )
        assert config.task_source.path == "tasks.jsonl"
        assert len(config.agents) == 2
        assert config.agents[0].framework == "openhands"
        assert config.agents[1].model == "gpt-4o"
        assert config.output_dir == "./custom_output"
        assert config.parallel_workers == 4

    def test_agent_config_defaults(self):
        """Test default agent configuration."""
        agent = AgentConfig()
        assert agent.framework == "openhands"
        assert agent.max_steps == 30
        assert agent.temperature == 0.0

    def test_sandbox_config_defaults(self):
        """Test default sandbox configuration."""
        sandbox = SandboxConfig()
        assert sandbox.timeout == 600
        assert sandbox.memory_limit == "4g"
        assert sandbox.network_enabled is False

    def test_reward_config_defaults(self):
        """Test default reward configuration."""
        reward = RewardConfig()
        assert reward.enable_rule_layer is True
        assert reward.enable_model_layer is False
        assert len(reward.rubric_weights) == 5
        assert abs(sum(reward.rubric_weights.values()) - 1.0) < 0.01


class TestPipeline:
    """Tests for Pipeline basic setup."""

    def test_pipeline_creation(self):
        """Test pipeline can be created with config."""
        config = PipelineConfig()
        pipeline = Pipeline(config)
        assert pipeline.config == config

    def test_run_single_returns_trajectory(self):
        """Test run_single returns a Trajectory."""
        config = PipelineConfig()
        pipeline = Pipeline(config)
        task = TaskInfo(task_id="test-001", description="Test task")
        agent_config = AgentConfig()

        trajectory = pipeline.run_single(task, agent_config)

        assert isinstance(trajectory, Trajectory)
        assert trajectory.task_id == "test-001"
        assert trajectory.agent_framework == "openhands"

    def test_pipeline_result_dataclass(self):
        """Test PipelineResult has correct fields."""
        result = PipelineResult(
            total_tasks=10,
            completed=8,
            failed=2,
            trajectories_path="/tmp/traj.jsonl",
        )
        assert result.total_tasks == 10
        assert result.completed == 8
        assert result.failed == 2
        assert result.trajectories_path == "/tmp/traj.jsonl"


class TestTrajectory:
    """Tests for Trajectory dataclass."""

    def test_default_trajectory(self):
        """Test default trajectory values."""
        traj = Trajectory()
        assert traj.task_id == ""
        assert traj.steps == []
        assert traj.reward == 0.0
        assert traj.success is False

    def test_custom_trajectory(self):
        """Test trajectory with custom values."""
        traj = Trajectory(
            task_id="task-001",
            agent_framework="openhands",
            agent_model="claude-sonnet-4-20250514",
            steps=[{"action": "file_read", "path": "/test.py"}],
            total_steps=1,
            success=True,
            reward=0.85,
            step_rewards=[0.85],
        )
        assert traj.task_id == "task-001"
        assert len(traj.steps) == 1
        assert traj.reward == 0.85
        assert traj.success is True



"""AgentTrajectoryHub - Agent 轨迹数据 Pipeline 编排层

串联 Sandbox -> Recorder -> Reward 全流程，产出可训练的数据集。
"""

import logging

__version__ = "0.1.0"

from trajectoryhub.collect import collect
from trajectoryhub.pipeline import Pipeline, PipelineConfig, PipelineResult, Trajectory
from trajectoryhub.exporter import DatasetExporter

logging.getLogger(__name__).addHandler(logging.NullHandler())

__all__ = [
    "Pipeline",
    "PipelineConfig",
    "PipelineResult",
    "Trajectory",
    "DatasetExporter",
    "collect",
    "__version__",
]



"""Pipeline configuration - 流水线配置."""

from typing import Any, Dict, List, Optional

from pydantic import BaseModel, Field


class TaskSource(BaseModel):
    """任务来源配置.

    Attributes:
        path: 任务定义文件路径 (JSONL 格式)
        source_type: 任务来源类型 (jsonl / swebench / custom)
        dataset_name: SWE-bench 数据集名称 (仅 swebench 类型使用)
        split: 数据集分割 (train / test / dev)
        limit: 限制加载的任务数量
        filters: 过滤条件 (language, difficulty, type 等)
    """

    path: Optional[str] = None
    source_type: str = "jsonl"
    dataset_name: Optional[str] = None
    split: str = "test"
    limit: Optional[int] = None
    filters: Dict[str, Any] = Field(default_factory=dict)


class AgentConfig(BaseModel):
    """Agent 配置.

    Attributes:
        framework: Agent 框架 (openhands / sweagent / custom)
        model: 使用的 LLM 模型 (e.g. claude-sonnet-4-20250514, gpt-4o)
        max_steps: 最大执行步数
        temperature: 采样温度
        extra_args: 传给 Agent 框架的额外参数
    """

    framework: str = "openhands"
    model: str = "claude-sonnet-4-20250514"
    max_steps: int = 30
    temperature: float = 0.0
    domain: str = "coding"
    extra_args: Dict[str, Any] = Field(default_factory=dict)


class SandboxConfig(BaseModel):
    """沙箱环境配置.

    Attributes:
        image: Docker 镜像名
        timeout: 单任务超时时间 (秒)
        memory_limit: 内存限制 (e.g. "4g")
        network_enabled: 是否允许网络访问
    """

    image: str = "knowlyr/sandbox:latest"
    timeout: int = 600
    memory_limit: str = "4g"
    network_enabled: bool = False


class RewardConfig(BaseModel):
    """Reward 计算配置.

    Attributes:
        enable_rule_layer: 启用规则层 (使用 data-check)
        enable_model_layer: 启用模型层 (LLM-as-Judge)
        judge_model: Judge 使用的 LLM 模型
        rubric_weights: Rubric 各维度的权重
            - goal_progress: 目标推进
            - tool_selection: 工具选择
            - param_correctness: 参数正确性
            - info_utilization: 信息利用
            - non_redundancy: 非冗余性
    """

    enable_rule_layer: bool = True
    enable_model_layer: bool = False
    judge_model: str = "claude-sonnet-4-20250514"
    rubric_weights: Dict[str, float] = Field(
        default_factory=lambda: {
            "goal_progress": 0.3,
            "tool_selection": 0.2,
            "param_correctness": 0.2,
            "info_utilization": 0.15,
            "non_redundancy": 0.15,
        }
    )


class PipelineConfig(BaseModel):
    """Pipeline 全局配置.

    Attributes:
        task_source: 任务来源配置
        agents: Agent 配置列表 (支持多 Agent 对比)
        sandbox_config: 沙箱环境配置
        reward_config: Reward 计算配置
        output_dir: 输出目录
        parallel_workers: 并行工作进程数
        checkpoint_interval: 每隔多少个任务做一次 checkpoint
        resume_from: 从 checkpoint 恢复时的路径
    """

    task_source: TaskSource = Field(default_factory=TaskSource)
    agents: List[AgentConfig] = Field(default_factory=lambda: [AgentConfig()])
    sandbox_config: SandboxConfig = Field(default_factory=SandboxConfig)
    reward_config: RewardConfig = Field(default_factory=RewardConfig)
    output_dir: str = "./output"
    parallel_workers: int = 1
    checkpoint_interval: int = 10
    resume_from: Optional[str] = None
    domain: str = "coding"



"""测试 Pipeline.run_from_log / run_batch_from_logs 集成."""

import json

import pytest

from trajectoryhub.config import PipelineConfig
from trajectoryhub.pipeline import Pipeline, Trajectory
from trajectoryhub.tasks import TaskInfo


# ── 测试数据工厂 ───────────────────────────────────────────────────


def _write_openhands_log(path, task_desc="修复 bug"):
    """写一个最小的合法 OpenHands JSONL 日志."""
    events = [
        {
            "id": 0, "source": "user", "action": "message",
            "args": {"content": task_desc, "task_id": "oh-test"},
            "timestamp": "2026-01-01T10:00:00Z",
        },
        {
            "id": 1, "source": "agent", "action": "run",
            "args": {"command": "pytest tests/"},
            "thought": "跑测试",
            "timestamp": "2026-01-01T10:01:00Z",
            "extras": {"model": "claude-sonnet-4-20250514", "agent_class": "CodeActAgent"},
        },
        {
            "id": 2, "observation": "run",
            "content": "1 passed",
            "extras": {"exit_code": 0},
        },
        {
            "id": 3, "source": "agent", "action": "edit",
            "args": {"path": "/workspace/fix.py"},
            "thought": "修复代码",
            "timestamp": "2026-01-01T10:02:00Z",
        },
        {
            "id": 4, "observation": "edit",
            "content": "File edited",
            "extras": {},
        },
        {
            "id": 5, "action": "finish",
            "args": {"outputs": {"test_result": {
                "success": True, "tests_passed": 2, "tests_failed": 0,
            }}},
        },
    ]
    with open(path, "w") as f:
        for e in events:
            f.write(json.dumps(e) + "\n")


def _write_sweagent_log(path, task_desc="修复 Django bug"):
    """写一个最小的合法 SWE-agent JSON 日志."""
    data = {
        "problem_statement": task_desc,
        "history": [
            [{"action": "bash", "command": "ls", "thought": "看看文件"}, "file.py\n"],
            [{"action": "edit", "path": "file.py"}, "File edited\n"],
            [{"action": "submit"}, "Submitted"],
        ],
        "info": {
            "instance_id": "swe-test-001",
            "model_name": "gpt-4o",
            "repo": "django/django",
            "base_commit": "abc123",
            "exit_status": "submitted",
            "model_stats": {"tokens_sent": 1000, "tokens_received": 500},
        },
    }
    with open(path, "w") as f:
        json.dump(data, f)


# ── run_from_log 测试 ──────────────────────────────────────────────


class TestRunFromLog:
    """验证 run_from_log 完整流程."""

    def test_openhands_log(self, tmp_path):
        """OpenHands 日志应产出带评分的 Trajectory."""
        log_path = tmp_path / "log.jsonl"
        _write_openhands_log(log_path)

        pipeline = Pipeline(PipelineConfig(output_dir=str(tmp_path)))
        traj = pipeline.run_from_log(log_path, "openhands")

        assert isinstance(traj, Trajectory)
        assert traj.task_id == "oh-test"
        assert traj.agent_framework == "openhands/CodeActAgent"
        assert traj.agent_model == "claude-sonnet-4-20250514"
        assert traj.total_steps == 2  # run + edit
        assert traj.success is True
        assert traj.reward > 0.0  # reward engine 应给正分
        assert len(traj.step_rewards) == 2

    def test_sweagent_log(self, tmp_path):
        """SWE-agent 日志应产出带评分的 Trajectory."""
        log_path = tmp_path / "traj.json"
        _write_sweagent_log(log_path)

        pipeline = Pipeline(PipelineConfig(output_dir=str(tmp_path)))
        traj = pipeline.run_from_log(log_path, "sweagent")

        assert isinstance(traj, Trajectory)
        assert traj.task_id == "swe-test-001"
        assert traj.agent_framework == "swe-agent"
        assert traj.agent_model == "gpt-4o"
        assert traj.total_steps == 3  # bash + edit + submit
        assert traj.success is True
        assert traj.reward > 0.0

    def test_swe_agent_alias(self, tmp_path):
        """swe-agent 别名应等同于 sweagent."""
        log_path = tmp_path / "traj.json"
        _write_sweagent_log(log_path)

        pipeline = Pipeline(PipelineConfig(output_dir=str(tmp_path)))
        traj = pipeline.run_from_log(log_path, "swe-agent")
        assert traj.agent_framework == "swe-agent"

    def test_custom_task_override(self, tmp_path):
        """传入 task 参数应覆盖日志中的 task 信息."""
        log_path = tmp_path / "log.jsonl"
        _write_openhands_log(log_path)

        custom_task = TaskInfo(task_id="custom-001", description="自定义任务")
        pipeline = Pipeline(PipelineConfig(output_dir=str(tmp_path)))
        traj = pipeline.run_from_log(log_path, "openhands", task=custom_task)

        assert traj.task_id == "custom-001"
        assert traj.metadata["task_description"] == "自定义任务"

    def test_step_details(self, tmp_path):
        """steps 应包含 thought/tool/params/output/exit_code."""
        log_path = tmp_path / "log.jsonl"
        _write_openhands_log(log_path)

        pipeline = Pipeline(PipelineConfig(output_dir=str(tmp_path)))
        traj = pipeline.run_from_log(log_path, "openhands")

        step = traj.steps[0]
        assert "thought" in step
        assert "tool" in step
        assert "params" in step
        assert "output" in step
        assert "exit_code" in step
        assert step["thought"] == "跑测试"
        assert step["tool"] == "bash"

    def test_metadata_includes_outcome(self, tmp_path):
        """metadata 应包含 outcome 信息."""
        log_path = tmp_path / "log.jsonl"
        _write_openhands_log(log_path)

        pipeline = Pipeline(PipelineConfig(output_dir=str(tmp_path)))
        traj = pipeline.run_from_log(log_path, "openhands")

        assert traj.metadata["outcome_tests_passed"] == 2
        assert traj.metadata["outcome_tests_failed"] == 0

    def test_unsupported_framework(self, tmp_path):
        """不支持的框架应抛出 ValueError."""
        log_path = tmp_path / "log.jsonl"
        _write_openhands_log(log_path)

        pipeline = Pipeline(PipelineConfig(output_dir=str(tmp_path)))
        with pytest.raises(ValueError, match="不支持的框架"):
            pipeline.run_from_log(log_path, "unknown_framework")

    def test_duration_tracked(self, tmp_path):
        """duration_seconds 应大于 0."""
        log_path = tmp_path / "log.jsonl"
        _write_openhands_log(log_path)

        pipeline = Pipeline(PipelineConfig(output_dir=str(tmp_path)))
        traj = pipeline.run_from_log(log_path, "openhands")

        assert traj.duration_seconds >= 0.0


# ── run_batch_from_logs 测试 ───────────────────────────────────────


class TestRunBatchFromLogs:
    """验证批量日志处理."""

    def test_batch_sweagent(self, tmp_path):
        """批量处理多个 SWE-agent 日志."""
        for i in range(3):
            _write_sweagent_log(tmp_path / f"traj_{i}.json", f"任务 {i}")

        pipeline = Pipeline(PipelineConfig(output_dir=str(tmp_path / "out")))
        trajectories = pipeline.run_batch_from_logs(tmp_path, "sweagent", "*.json")

        assert len(trajectories) == 3
        for traj in trajectories:
            assert isinstance(traj, Trajectory)
            assert traj.agent_framework == "swe-agent"
            assert traj.reward > 0.0

    def test_batch_empty_dir(self, tmp_path):
        """空目录应返回空列表."""
        pipeline = Pipeline(PipelineConfig(output_dir=str(tmp_path / "out")))
        trajectories = pipeline.run_batch_from_logs(tmp_path, "sweagent", "*.json")
        assert trajectories == []

    def test_batch_unsupported_framework(self, tmp_path):
        """不支持的框架应抛出 ValueError."""
        pipeline = Pipeline(PipelineConfig(output_dir=str(tmp_path / "out")))
        with pytest.raises(ValueError, match="不支持的框架"):
            pipeline.run_batch_from_logs(tmp_path, "unknown")


# ── Pipeline 可选依赖标志测试 ──────────────────────────────────────


class TestOptionalDeps:
    """验证可选依赖检测."""

    def test_has_recorder(self):
        """在开发环境中 recorder 应可用."""
        from trajectoryhub.pipeline import _HAS_RECORDER
        assert _HAS_RECORDER is True

    def test_has_reward(self):
        """在开发环境中 reward 应可用."""
        from trajectoryhub.pipeline import _HAS_REWARD
        assert _HAS_REWARD is True

    def test_adapter_registry_populated(self):
        """适配器注册表应包含 openhands 和 sweagent."""
        from agentrecorder.adapters import get_adapter, list_adapters
        assert "openhands" in list_adapters()
        assert "sweagent" in list_adapters()
        assert "swe-agent" in list_adapters()
        assert get_adapter("openhands") is not None
        assert get_adapter("sweagent") is not None
